@@ -18,8 +18,8 @@
 
 #include "/sys/wizlist.h"
 #include "/sys/driver_hook.h"
-#include "/sys/functionlist.h"
 #include "/sys/objectinfo.h"
+#include "/sys/functionlist.h"
 #include "/sys/erq.h"
 
 #define INIT_FILE "/room/init_file"
@@ -66,11 +66,11 @@ int query_player_level (string what); // forward
 #ifndef __COMPAT_MODE__
 
 //---------------------------------------------------------------------------
-string file_name(object ob)
+string object_name(object ob)
 {
     string rc;
 
-    rc = efun::file_name(ob);
+    rc = efun::object_name(ob);
     return stringp(rc) ? rc[1..] : 0;
 }
 
@@ -132,44 +132,72 @@ static void _move_hook_fun (object item, object dest)
 {
   object *others;
   int i;
+  string name;
 
   /* PLAIN:
   if (item != this_object)
       raise_error("Illegal to move other object than this_object()\n");
   */
 
-  /* PLAIN: the call to exit() is needed in compat mode only */
   if (living(item) && environment(item))
   {
+      name = query_once_interactive(item) ? item->query_real_name()
+                                          : object_name(item);
+      /* PLAIN: the call to exit() is needed in compat mode only */
       efun::set_this_player(item);
-      environment(item)->exit(item);
+      object env = environment(item);
+      env->exit(item);
+      if (!item)
+          raise_error(sprintf("%O->exit() destructed item %s before move.\n"
+                             , env, name));
   }
+  else
+      name = object_name(item);
+
+  /* This is the actual move of the object. */
 
   efun::set_environment(item, dest);
+
+  /* Moving a living object will cause init() to be called in the new
+   * environment.
+   */
   if (living(item)) {
     efun::set_this_player(item);
     dest->init();
     if (!item)
-      raise_error(sprintf("%O->init() destructed moved item\n", dest));
+      raise_error(sprintf("%O->init() destructed moved item %s\n", dest, name));
     if (environment(item) != dest)
       return;
   }
-  others = all_inventory(dest);
-  others[member(others, item)] = 0;
-  for (i = 0; i < sizeof(others); i++)
+
+  /* Call init() in item once foreach living object in the new environment
+   * but only if the item is (still) in the same environment.
+   */
+  others = all_inventory(dest) - ({ item });
+  foreach (object obj : others)
   {
-    if (living(others[i])) {
-      efun::set_this_player(others[i]);
+    if (living(obj) && environment(obj) == environment(item)) {
+      efun::set_this_player(obj);
       item->init();
     }
     if (!item)
-      raise_error(sprintf("item->init() for %O (#%d) destructed moved item\n", others[i], i));
+      raise_error(sprintf("item->init() for %O destructed moved item %s\n", obj, name));
   }
+
+  /* Call init() in each of the  objects themselves, but only if item
+   * didn't move away already.
+   */
   if (living(item)) {
-    efun::set_this_player(item);
-    filter_objects(others, "init");
+    foreach (object obj : others)
+    {
+        efun::set_this_player(item); // In case something new was cloned
+        if (environment(obj) == environment(item))
+            obj->init();
+    }
   }
-  if (living(dest) && item) {
+
+  /* If the destination is alive as well, call item->init() for it. */
+  if (living(dest) && item && environment(item) == dest) {
     efun::set_this_player(dest);
     item->init();
   }
@@ -341,7 +369,7 @@ void inaugurate_master (int arg)
 // mudwho or wizlist handling has to be initialized here.
 //
 // Besides that, do whatever you feel you need to do,
-// e.g. set_auto_include_string(), or give the master a decent euid.
+// e.g. set_driver_hook(), or give the master a decent euid.
 
 {
     if (!arg) {
@@ -547,8 +575,8 @@ mixed get_simul_efun ()
 // Load the simul_efun object(s) and return one or more paths of it.
 //
 // Result:
-//   Either a single string with the file_name() of the simul_efun object,
-//   or an array of strings which has to start with that file_name().
+//   Either a single string with the object_name() of the simul_efun object,
+//   or an array of strings which has to start with that object_name().
 //   Return 0 if this feature isn't wanted.
 //
 // Note that the object(s) must be loaded by this function!
@@ -1176,7 +1204,7 @@ void runtime_error ( string err, string prg, string curobj, int line
 // trouble prematurely) and give root objects only the permission to
 // execute the real efuns.
 //
-// See also creator_file(), valid_read() and valid_write().
+// See also valid_read() and valid_write().
 //===========================================================================
 
 //---------------------------------------------------------------------------
@@ -1207,10 +1235,7 @@ int privilege_violation (string op, mixed who, mixed arg, mixed arg2)
 //                       is shadowed by a 'nomask'-type simul_efun.
 //   rename_object     : The current object <who> renames object <arg>
 //                       to name <arg2>.
-//   send_imp          : Send UDP-data to host arg3 (deprecated).
-//   send_udp          : Send UDP-data to host <arg>.
-//   set_auto_include_string : Set the string automatically included by
-//                       the compiler (deprecated).
+//   send_imp          : Send UDP-data to host <arg>.
 //   get_extra_wizinfo : Get the additional wiz-list info for wizard <arg>.
 //   set_extra_wizinfo : Set the additional wiz-list info for wizard <arg>.
 //   set_extra_wizinfo_size : Set the size of the additional wizard info
@@ -1262,6 +1287,7 @@ int privilege_violation (string op, mixed who, mixed arg, mixed arg2)
       default:
 	return -1; /* Make this violation an error */
     }
+    return 0;
 }
 
 //---------------------------------------------------------------------------
@@ -1317,6 +1343,7 @@ int query_player_level (string what)
     case "error messages":
 	return level >= 20;
     }
+    return 0;
 }
 
 //---------------------------------------------------------------------------
@@ -1346,6 +1373,7 @@ int valid_trace (string what)
     case "traceprefix":
 	return level >= 24;
     }
+    return 0;
 }
 
 //---------------------------------------------------------------------------
@@ -1355,7 +1383,7 @@ int valid_exec (string name, object ob, object obfrom)
 //
 // Arguments:
 //    name  : The name of the _program_ attempting to rebind the connection.
-//            This is not the file_name() of the object, and has no leading
+//            This is not the object_name() of the object, and has no leading
 //            slash.
 //    ob    : The object to receive the connection.
 //    obfrom: The object giving the connection away.
@@ -1550,6 +1578,7 @@ mixed valid_read  (string path, string euid, string fun, object caller)
      * recognized, we come here.
      * The default returned zero indicates deny of access.
      */
+    return 0;
 }
 
 //---------------------------------------------------------------------------
@@ -1980,9 +2009,10 @@ void save_wiz_file()
  */
 
 {
-    rm("/WIZLIST");
+#ifdef __WIZLIST__
+    rm(__WIZLIST__);
     write_file(
-      "/WIZLIST",
+      __WIZLIST__,
       implode(
         map(wizlist_info(),
           lambda(({'a}),
@@ -1995,6 +2025,7 @@ void save_wiz_file()
         ), ""
       )
     );
+#endif
 }
 
 //---------------------------------------------------------------------------
@@ -2069,4 +2100,3 @@ string master_create_wizard(string owner, string domain, object caller)
 }
 
 /****************************************************************************/
-
