@@ -54,7 +54,7 @@
 // object connect ()
 //   Handle the request for a new connection.
 //
-// void disconnect (object obj)
+// void disconnect (object obj, string remaining)
 //   Handle the loss of an IP connection.
 //
 // void remove_player (object player)
@@ -77,7 +77,7 @@
 // mixed inherit_file (string file, string compiled_file)
 //   Return the full pathname for an inherited object.
 //
-// string object_name (object obj)
+// string printf_obj_name (object obj)
 //   Return a printable name for an object.
 //
 // mixed prepare_destruct (object obj)
@@ -87,7 +87,6 @@
 //   Handle quotas in times of memory shortage.
 //
 // void receive_udp (string host, string msg, int port)
-// void receive_imp (string host, string msg, int port) // deprecated
 //   Handle a received UDP message.
 //
 // void slow_shut_down (int minutes)
@@ -106,12 +105,18 @@
 //   Announce a compiler-time error or warning.
 //
 // mixed heart_beat_error (object culprit, string err,
-//                         string prg, string curobj, int line)
+//                         string prg, string curobj, int line,
+//                         int caught )
 //   Announce an error in the heart_beat() function.
 //
-// void runtime_error (string err, string prg, string curobj, int line)
+// void runtime_error (string err, string prg, string curobj, int line
+//                    , mixed culprit, int caught)
 //   Announce a runtime error.
 //
+// void runtime_warning (string msg, string curobj, string prg, int line
+//                      , int inside_catch)
+//   Announce a runtime warning.
+//
 //---------------------------------------------------------------------------
 //     Security and Permissions
 //
@@ -254,7 +259,7 @@ void inaugurate_master (int arg)
 // mudwho or wizlist handling has to be initialized here.
 //
 // Besides that, do whatever you feel you need to do,
-// e.g. set_auto_include(), or give the master a decent euid.
+// e.g. set_driver_hook(), or give the master a decent euid.
 
 
 //---------------------------------------------------------------------------
@@ -412,15 +417,22 @@ object connect ()
 //
 // The gamedriver will call the lfun 'logon()' in the login object after
 // binding the connection to it. That lfun has to return !=0 to succeed.
+//
+// If connect() initiates a secure connection without setting a callback,
+// and the connection is still handshaking at the time connect() returns,
+// the driver will delay the call to logon() until the handshake either
+// succeeds or fails.
+
 
 
 //---------------------------------------------------------------------------
-void disconnect (object obj)
+void disconnect (object obj, string remaining)
 
 // Handle the loss of an IP connection.
 //
 // Argument:
 //   obj: The (formerly) interactive object (player).
+//   remaining: The remaining unprocessed input data from the connection.
 //
 // This called by the gamedriver to handle the removal of an IP connection,
 // either because the connection is already lost ('netdeath') or due to
@@ -584,6 +596,7 @@ mixed prepare_destruct (object obj)
 // lfun 'remove()' in the object, which then calls the efun destruct()
 // after performing all the adjustments.
 
+
 //---------------------------------------------------------------------------
 void quota_demon (void)
 
@@ -601,7 +614,6 @@ void quota_demon (void)
 
 //---------------------------------------------------------------------------
 void receive_udp (string host, string msg, int port)
-void receive_imp (string host, string msg, int port) // deprecated
 
 // Handle a received UDP message.
 //
@@ -611,10 +623,6 @@ void receive_imp (string host, string msg, int port) // deprecated
 //   port: the port number from which the message was sent.
 //
 // This function is called for every message received on the UDP port.
-//
-// The driver first calls receive_udp(). If that method doesn't exist
-// and if the driver is compiled with USE_DEPRECATED, it will then
-// call receive_imp().
 
 
 //---------------------------------------------------------------------------
@@ -730,7 +738,8 @@ void log_error (string file, string err, int warn)
 
 //---------------------------------------------------------------------------
 mixed heart_beat_error (object culprit, string err,
-                        string prg, string curobj, int line)
+                        string prg, string curobj, int line,
+                        int caught)
 
 // Announce an error in the heart_beat() function.
 //
@@ -740,6 +749,7 @@ mixed heart_beat_error (object culprit, string err,
 //   prg    : The executed program (might be 0).
 //   curobj : The object causing the error (might be 0).
 //   line   : The line number where the error occured (might be 0).
+//   caught : 0 if the error is not caught, != 0 if it is caught.
 //
 // Result:
 //   Return anything != 0 to restart the heart_beat in culprit.
@@ -757,7 +767,7 @@ mixed heart_beat_error (object culprit, string err,
 
 //---------------------------------------------------------------------------
 void runtime_error (string err, string prg, string curobj, int line
-                   , mixed culprit)
+                   , mixed culprit, int caught)
 
 // Announce a runtime error.
 //
@@ -768,6 +778,7 @@ void runtime_error (string err, string prg, string curobj, int line
 //   line   : The line number where the error occured.
 //   culprit: -1 for runtime errors; the object holding the heart_beat()
 //            function for heartbeat errors.
+//   caught : 0 if the error is not caught, != 0 if it is caught.
 //
 // This function has to announce a runtime error to the active user,
 // resp. handle a runtime error which occured during the execution of
@@ -786,6 +797,51 @@ void runtime_error (string err, string prg, string curobj, int line
 // Note that <prg> denotes the program actually executed (which might be
 // inherited) whereas <curobj> is just the offending object for which the
 // program was executed.
+//
+// One common pitfall in the implementation of runtime_error() is that
+// runtime_error() itself could run out of evaluation ticks, causing a
+// runtime error itself. The workaround is to use limited() like this:
+//
+//   static void
+//   handle_runtime_error (string err, string prg, string curobj, int line)
+//   { ... the actual error handler ... }
+//
+//   static void
+//   call_runtime_error (string err, string prg, string curobj, int line)
+//   {
+//       limited(#'handle_runtime_error, ({ 200000 }), err, prg, curobj, line);
+//   }
+//
+//   void
+//   runtime_error (string err, string prg, string curobj, int line)
+//   {
+//       limited(#'call_runtime_error, ({ LIMIT_UNLIMITED })
+//              , err, prg, curobj, line);
+//   }
+
+
+//---------------------------------------------------------------------------
+void runtime_warning (string msg, string curobj, string prg, int line
+                     , int inside_catch)
+
+// Announce a runtime warning.
+//
+// Arguments:
+//   err    : The warning message.
+//   curobj : The object causing the warning, may be 0.
+//   prg    : The executed program, may be 0.
+//   line   : The line number where the warning occured.
+//   inside_catch : != 0 if the warning occurs inside a catch().
+//
+// This function is to allow the mudlib to handle runtime warnings, for
+// example to log them into a database.
+//
+// Note that <prg> denotes the program actually executed (which might be
+// inherited) whereas <curobj> is just the offending object for which the
+// program was executed.
+// 
+// The driver imposes a limit of three nested warnings, to prevent endless
+// recursions.
 
 
 //===========================================================================
@@ -796,7 +852,7 @@ void runtime_error (string err, string prg, string curobj, int line
 // trouble prematurely) and give root objects only the permission to
 // execute the real efuns.
 //
-// See also creator_file(), valid_read() and valid_write().
+// See also valid_read() and valid_write().
 //===========================================================================
 
 //---------------------------------------------------------------------------
@@ -824,18 +880,16 @@ int privilege_violation (string op, mixed who, mixed arg, mixed arg2)
 //   enable_telnet     : Enable/disable telnet (<arg2>) for object <arg>.
 //   execute_command   : Execute command string <arg2> for the object <arg>.
 //   input_to          : Object <who> issues an 'ignore-bang'-input_to() for
-//                       commandgiver <arg2>; the exact flags are <arg2>.
-//   mysql:            : Object <who> attempted to execute mySQL efun <arg>.
+//                       commandgiver <arg>; the exakt flags are <arg2>.
+//   mysql             : Object <who> attempted to execute mySQL efun <arg>.
+//   pgsql             : Object <who> attempted to execute Postgres efun <arg>.
 //   net_connect       : Attempt to open a connection to host <arg>,
 //                        port <arg2>.
 //   nomask simul_efun : Attempt to get an efun <arg> via efun:: when it
 //                       is shadowed by a 'nomask'-type simul_efun.
 //   rename_object     : The current object <who> renames object <arg>
 //                       to name <arg2>.
-//   send_imp          : Send UDP-data to host <arg> (deprecated).
 //   send_udp          : Send UDP-data to host <arg>.
-//   set_auto_include_string : Set the string automatically included by
-//                       the compiler (deprecated).
 //   get_extra_wizinfo : Get the additional wiz-list info for wizard <arg>.
 //   set_extra_wizinfo : Set the additional wiz-list info for wizard <arg>.
 //   set_extra_wizinfo_size : Set the size of the additional wizard info
@@ -843,8 +897,9 @@ int privilege_violation (string op, mixed who, mixed arg, mixed arg2)
 //   set_driver_hook   : Set hook <arg> to <arg2>.
 //   set_max_commands  : Set the max. number of commands interactive
 //                       object <arg> can issue per second to <arg2>.
-//   limited:          : Execute <arg> with reduced/changed limits.
-//   set_limits        : Set limits to <arg>.
+//   limited           : Execute <arg> with reduced/changed limits
+//                       <arg2> (as return by query_limits()).
+//   set_limits        : Set limits to <arg> (as returned by query_limits()).
 //   set_this_object   : Set this_object() to <arg>.
 //   shadow_add_action : Add an action to function <arg2> of object <arg>
 //                       from the shadow <who> which is shadowing <arg>.
@@ -862,7 +917,7 @@ int privilege_violation (string op, mixed who, mixed arg, mixed arg2)
 // wizlist field. While a toplevel array, if found, will be copied, this does
 // not apply to nested arrays or to any mappings. You might also have some
 // sensitive closures there.
-// send_udp() should be watched as it could be abused to mess up the IMP.
+// send_udp() should be watched as it could be abused.
 // The xxx_extra_wizinfo operations are necessary for a proper wizlist and
 // should therefore be restricted to admins.
 // All other operations are potential sources for direct security breaches -
@@ -1048,6 +1103,7 @@ mixed valid_write (string path, string euid, string fun, object caller)
 //   rename_from         (for each the old name of a rename())
 //   rename_to           (for the new name of a rename())
 //   mkdir
+//   memdump
 //   objdump
 //   opcdump
 //   save_object
