@@ -7,9 +7,12 @@
 // to RFC 1143 and so follows all rules of the telnet protocol RFC 854.
 // The driver itself is not quite compliant.
 //
+// Find the newest version of this file at
+//   http://wl.mud.de/mud/doc/wwwstd/standardobjekte.html
+// 
 // $Log: telnetneg.c,v $
-// Revision 1.22  2002/12/09 14:45:19  Fiona
-// added explanation on repeated SB TTYPE START
+// Revision 1.25  2003/07/07 07:08:52  Fiona
+// LM_SLC more save against protocol violations
 //
 
 #pragma strict_types
@@ -30,14 +33,14 @@
 // machine as suggested with RFC 1143.
 //
 // The machine is used with three functions:
-//   set_telnet()    requests a change in the telnet state
+//   set_telnet()    requests a change in the telnet state 
 //   set_callback()  sets our prefered telnet states and a callback
 //                   function which is called on telnet state changes
 //   query_telnet()  query state and sb info
 //
 // The driver communicates with the engine through the H_TELNET_NEG
 // and got_telnet().
-//
+// 
 // Do this in logon() to turn IAC quoting on:
 //   set_connection_charset(({255})*32, 1);
 //
@@ -50,7 +53,7 @@
 nosave mapping ts; // Complete telnet negotation state
 
 // Set preferences and callbacks
-//
+// 
 // r_a_cb is the preference for the state on the remote side. It could be
 //   DO, DONT or a callback closure which decides if we get a request.
 // l_a_cb is the same for the local side.
@@ -129,7 +132,7 @@ static int set_telnet(int command, int option) {
           } else {
 	    // ignore, request sent already
           }
-          break;
+          break; 
       }
       break;
 
@@ -186,7 +189,7 @@ static int set_telnet(int command, int option) {
           } else {
 	    // ignore, request sent already
           }
-          break;
+          break; 
       }
       break;
   }
@@ -216,7 +219,7 @@ void got_telnet(int command, int option, int *optargs) {
   // Make shure it's a driver apply. Hope this never breaks ;)
   if (caller_stack_depth()) return;
 
-
+  
   log = ts[TS_EXTRA, TSE_LOG];
   if (strlen(log) > 4000) log = "..." + log[<3800..];
   log += "got  " + telnet_to_text(command, option, optargs) + "\n";
@@ -383,7 +386,7 @@ void got_telnet(int command, int option, int *optargs) {
     agree = ts[option, TS_SBCB];
     if (closurep(agree)) funcall(agree, command, option, optargs);
     break;
-
+      
   }
 
   // Do full negotation only if client can telnetneg, means we got some
@@ -431,7 +434,7 @@ private int send(int* x) {
         y[i..i+1] = ({ 0xff });
         --j;
       }
-    }
+    } 
   }
 
   log = ts[TS_EXTRA, TSE_LOG];
@@ -490,21 +493,21 @@ private string telnet_to_text(int command, int option, int* args) {
           if (i > j) return d_txt;
           args = args[i-3..]; // dump rest (is error)
           break;
-
+          
       }
 
     }
 
     if (command == SB && option != TELOPT_NAWS && option != TELOPT_LINEMODE) {
       d_txt += " " + TELQUAL2STRING(args[0]);
-      if (sizeof(args) > 1) d_txt += " (" +
+      if (sizeof(args) > 1) d_txt += " (" + 
         implode(map(args[1..], (: sprintf("%02x", $1) :)), ",") + ")";
     }
     else if (sizeof(args)) d_txt +=
       " (" + implode(map(args, (: sprintf("%02x", $1) :)), ",") + ")";
   }
   return d_txt;
-}
+}    
 
 // Full negotiation tries to set all our preferences
 // Options with closures as preference are not actively enabled.
@@ -543,7 +546,7 @@ private void tel_error(string err) {
 //
 // Here is defined how the mud will act in the negotiation.
 
-int* tm_t;
+nosave int* tm_t;
 
 void create() {
   if (!ts) {
@@ -578,7 +581,7 @@ void set_noecho(int flag) {
   //
   //   void noecho_hook(int flag, int ob) {
   //     if (ob) ob->set_noecho(flag);
-  //   }
+  //   } 
   int i, j;
 
   if (object_name(previous_object()) != __MASTER_OBJECT__) return;
@@ -748,7 +751,7 @@ private void start_lm(int command, int option) {
   send(({ IAC, SB, TELOPT_LINEMODE, DO,
     LM_FORWARDMASK, IAC, 0xff, IAC, 0xff, IAC, 0xff, IAC, 0xff, IAC, SE }));
   ts[option, TS_SB][LM_FORWARDMASK] = WANT_YES;
-
+  
 }
 
 private void start_eor(int command, int option) {
@@ -948,7 +951,7 @@ private void sb_ttype(int command, int option, int* optargs) {
   // some time, especially on slow links. MS windows 2000's and XP's
   // telnet have an emulation with key on-off sequences rather that chars.
   // Commands typed while TTYPE is negotiated may be wrong with that clients.
-
+ 
   i = member(all[1], value);
   if (i < 0) {
     all[1] += ({ value });
@@ -965,7 +968,7 @@ private void sb_ttype(int command, int option, int* optargs) {
       case 1:
         if (Q_REMOTE(ts[option, TS_STATE]) == YES) start_sb(WILL, option);
         break; // just retry
-      case 2:
+      case 2: 
         tel_error("could not get initial TTYPE, may be rfc930 client");
         if (value != "vtnt") break;
         tel_error("trying trick to restart TTYPE list on windows client");
@@ -989,7 +992,7 @@ private void sb_ttype(int command, int option, int* optargs) {
     set_telnet(DO, TELOPT_BINARY);
 }
 
-private void sb_tspeed(int command, int option, int* optargs) {
+private void sb_tspeed(int command, int option, int* optargs) {            
   int i, j, os;
 
   os = sizeof(optargs);
@@ -1037,7 +1040,7 @@ private void sb_env(int command, int option, int* optargs) {
       ++j;
       continue;
     }
-    if (optargs[j] < ENV_USERVAR) {
+    if (optargs[j] <= ENV_USERVAR) {
       if (i != j) {
         s = to_string(filter(optargs[i..j-1], #'>=, ' '));
         if (strlen(s) != j-i)
@@ -1055,12 +1058,12 @@ private void sb_env(int command, int option, int* optargs) {
       tel_error("SB ENV contained bad characters");
     mix += ({ s });
   }
-
+        
   // Due to some weird implementation mistakes (see RFC 1571)
   // we can't be shure if VALUE or VAR are valid or swapped.
   // So we don't distuingush VAR and USERVAR and assume that every
   // variable's name is followed by its value.
-
+        
   s = 0;
   env = ([]);
   os = sizeof(mix);
@@ -1144,7 +1147,7 @@ private void sb_line(int command, int option, int* optargs) {
       }
       state = ts[option, TS_SB][LM_FORWARDMASK];
       switch (state) {
-        case NO:
+        case NO: 
           tel_error("got spontan WILL FORWARDMASK (denied)");
           break;
         case YES:
@@ -1165,7 +1168,7 @@ private void sb_line(int command, int option, int* optargs) {
       }
       state = ts[option, TS_SB][LM_FORWARDMASK];
       switch (state) {
-        case YES:
+        case YES: 
           send(({ IAC, SB, option, DONT, LM_FORWARDMASK, IAC, SE }));
           ts[option, TS_SB][LM_FORWARDMASK] = NO;
           break;
@@ -1205,7 +1208,7 @@ private void sb_line(int command, int option, int* optargs) {
         // We dont want any of the session control keys (Synch - Susp)
         // Everything else is just acknoledged
         optargs = optargs[1..];
-        j = sizeof(optargs);
+        j = sizeof(optargs) - 3;
         for (i = 0; i < j; i+=3) {
           if (optargs[i+1] & SLC_ACK) {
             m_add(ts[option, TS_SB][LM_SLC], optargs[i],
@@ -1234,7 +1237,7 @@ private void sb_line(int command, int option, int* optargs) {
 void send_telopt_tm() {
   tm_t = utime();
   set_telnet(DO, TELOPT_TM);
-}
+}   
 
 void _dumptelnegs() {
   printf(ts[TS_EXTRA, TSE_LOG]);
