@@ -34,15 +34,18 @@
 #include "gcollect.h"
 #include "interpret.h"
 #include "main.h"
+#include "mstrings.h"
 #include "object.h"
 #include "simulate.h"
-#include "stralloc.h"
+#include "stdstrings.h"
 #include "strfuns.h"
 #include "svalue.h"
 #include "swap.h"
 #include "wiz_list.h"
 #include "xalloc.h"
 
+#include "i-eval_cost.h"
+
 #include "../mudlib/sys/debug_info.h"
 
 /*-------------------------------------------------------------------------*/
@@ -53,9 +56,9 @@
    */
 
 struct call {
+    struct call *next;   /* link to next structure */
     /* TODO: p_int or mp_int */ int delta;           /* Delay in relation to the previous structure */
     callback_t fun;
-    struct call *next;   /* link to next structure */
     object_t *command_giver;  /* the saved command_giver */
 };
 
@@ -101,7 +104,8 @@ insert_call (struct call *cop, int delay)
     for (copp = &call_list; NULL != (cop2 = *copp); copp = &cop2->next)
     {
         int delta;
-        if ((delta = cop2->delta) >= delay)
+        /* Insert the call_out *after* calls with the same delay (FIFO). */
+        if ((delta = cop2->delta) > delay)
         {
             cop2->delta -= delay;
             cop->delta = delay;
@@ -121,7 +125,7 @@ insert_call (struct call *cop, int delay)
 
 /*-------------------------------------------------------------------------*/
 svalue_t *
-new_call_out (svalue_t *sp, short num_arg)
+v_call_out (svalue_t *sp, int num_arg)
 
 /* EFUN: call_out()
  *
@@ -142,20 +146,6 @@ new_call_out (svalue_t *sp, short num_arg)
 
     arg = sp - num_arg + 1;
 
-    /* Test if the expected arguments are on the stack */
-
-    if (arg[0].type != T_STRING && arg[0].type != T_CLOSURE)
-    {
-        bad_efun_vararg(1, sp);
-        /* NOTREACHED */
-    }
-
-    if (arg[1].type != T_NUMBER)
-    {
-        bad_efun_vararg(2, sp);
-        /* NOTREACHED */
-    }
-
     /* If the current object is destructed, free everything on the stack
      * and return.
      */
@@ -170,7 +160,7 @@ new_call_out (svalue_t *sp, short num_arg)
 
     if (max_callouts && max_callouts <= num_callouts)
     {
-        error("Too many callouts at once (max. %ld).\n", (long)max_callouts);
+        errorf("Too many callouts at once (max. %ld).\n", (long)max_callouts);
         /* NOTREACHED */
     }
 
@@ -187,7 +177,7 @@ new_call_out (svalue_t *sp, short num_arg)
     if (arg[0].type == T_STRING)
     {
         error_index = setup_function_callback(&(cop->fun), current_object
-                                             , arg[0].u.string
+                                             , arg[0].u.str
                                              , num_arg-2, arg+2
                                              , MY_TRUE
                                              );
@@ -202,8 +192,9 @@ new_call_out (svalue_t *sp, short num_arg)
     if (error_index >= 0)
     {
         pfree(cop); /* The callback structure was invalidated automatically. */
-        bad_efun_vararg(error_index+2, arg - 1);
+        vefun_bad_arg(error_index+2, arg-1);
         /* NOTREACHED */
+        return arg-1;
     }
 
     /* We can do the callout.
@@ -211,43 +202,34 @@ new_call_out (svalue_t *sp, short num_arg)
 
     cop->command_giver = command_giver; /* save current player context */
     if (command_giver)
-        ref_object(command_giver, "new_call_out");  /* Bump its ref */
+        ref_object(command_giver, "f_call_out");  /* Bump its ref */
 
     /* Adjust the stack and get the delay */
     sp = arg - 1;
     delay = arg[1].u.number;
-    if (delay < 1)
-        delay = 1;
+    if (delay < 0)
+        delay = 0;
 
     /* Insert the new structure at its proper place in the list */
 
     insert_call(cop, delay);
 
     return sp;
-} /* new_call_out() */
+} /* v_call_out() */
+
 
 /*-------------------------------------------------------------------------*/
 void
-call_out (void)
+next_call_out_cycle (void)
 
-/* Check if there is any callout due to be called. If yes, do so.
- * This function is called from the heart_beat handling in the backend.c.
- * It sets up its own error recovery context so that errors during an
- * execution won't disturb the rest of the game.
+/* Starts the next call_out cycle by decrementing the first delta.
+ * This function is called in the backend cycle before heart_beats are handled.
  */
 
 {
     static int last_time;
       /* Last time this function was called */
 
-    static struct call *current_call_out;
-      /* Current callout, static so that longjmp() won't clobber it. */
-
-    static object_t *called_object;
-      /* Object last called, static so that longjmp() won't clobber it */
-
-    struct error_recovery_info error_recovery_info;
-
     /* No calls pending: just update the last_time and return */
 
     if (call_list == NULL)
@@ -260,19 +242,45 @@ call_out (void)
     if (last_time == 0)
         last_time = current_time;
 
-    /* Update the first .delta in the list (so it won't happen
-     * twice in case of an error.
+    /* Update the first .delta in the list.
      */
     call_list->delta -= current_time - last_time;
 
     last_time = current_time;
+} /* next_call_out_cycle() */
+
+
+/*-------------------------------------------------------------------------*/
+void
+call_out (void)
+
+/* Check if there is any callout due to be called. If yes, do so.
+ * This function is called from the heart_beat handling in the backend.c.
+ * It sets up its own error recovery context so that errors during an
+ * execution won't disturb the rest of the game.
+ */
+
+{
+    static struct call *current_call_out;
+      /* Current callout, static so that longjmp() won't clobber it. */
+
+    static object_t *called_object;
+      /* Object last called, static so that longjmp() won't clobber it */
+
+    struct error_recovery_info error_recovery_info;
+
+    /* No calls pending: fine. */
+
+    if (call_list == NULL)
+        return;
+
     current_interactive = NULL;
 
     /* Activate the local error recovery context */
 
     error_recovery_info.rt.last = rt_context;
     error_recovery_info.rt.type = ERROR_RECOVERY_BACKEND;
-    rt_context = (rt_context_t *)&error_recovery_info;
+    rt_context = (rt_context_t *)&error_recovery_info.rt;
 
     if (setjmp(error_recovery_info.con.text))
     {
@@ -282,6 +290,7 @@ call_out (void)
         object_t *ob;
         wiz_list_t *user;
 
+        mark_end_evaluation();
         clear_state();
         debug_message("%s Error in call out.\n", time_stamp());
         cop = current_call_out;
@@ -340,7 +349,8 @@ call_out (void)
          && load_ob_from_swap(ob) < 0)
         {
             debug_message("%s Error in call_out: out of memory: "
-                          "unswap object '%s'.\n", time_stamp(), ob->name);
+                          "unswap object '%s'.\n", time_stamp()
+                         , get_txt(ob->name));
             free_call(cop);
             continue;
         }
@@ -356,7 +366,7 @@ call_out (void)
             debug_message("%s call_out: user '%s' had an error: "
                           "rescheduling call_out.\n"
                          , time_stamp()
-                         , user->name ? user->name : "<null>");
+                         , user->name ? get_txt(user->name) : "<null>");
             insert_call(cop, 1);
             continue;
         }
@@ -419,7 +429,6 @@ call_out (void)
          */
 
         called_object = current_object = ob;
-        user = ob->user;
         if (user->last_call_out != current_time)
         {
             user->last_call_out = current_time;
@@ -428,8 +437,10 @@ call_out (void)
         else
             assigned_eval_cost = eval_cost = user->call_out_cost;
 
+        mark_start_evaluation();
         (void)backend_callback(&(cop->fun), 0);
         user->call_out_cost = eval_cost;
+        mark_end_evaluation();
 
         /* The function call used up all the arguments, now free
          * the rest
@@ -442,7 +453,7 @@ call_out (void)
 } /* call_out() */
 
 /*-------------------------------------------------------------------------*/
-void
+static void
 find_call_out (object_t *ob, svalue_t *fun, Bool do_free_call)
 
 /* Find the (first) callout for <ob>/<fun> (or <fun> if it is a closure).
@@ -455,97 +466,58 @@ find_call_out (object_t *ob, svalue_t *fun, Bool do_free_call)
 {
     struct call **copp, *cop;
     int delay = 0;
-    char *fun_name;
+    string_t *fun_name;
 
     /* Find callout by closure */
 
     if (fun->type != T_STRING)
     {
-        ph_int type;
-
         if (fun->type == T_CLOSURE)
         {
-            if (!CLOSURE_MALLOCED(type = fun->x.closure_type)
-             && type >= CLOSURE_EFUN)
+            for (copp = &call_list; NULL != (cop = *copp); copp = &cop->next)
             {
-                for (copp = &call_list; NULL != (cop = *copp); copp = &cop->next)
+                delay += cop->delta;
+                if (cop->fun.is_lambda
+                 && closure_eq(&(cop->fun.function.lambda), fun)
+                   )
                 {
-                    delay += cop->delta;
-                    if (cop->fun.is_lambda
-                     && cop->fun.function.lambda.x.closure_type == type
-                     && cop->fun.function.lambda.u.ob == ob)
-                    {
-                        goto found;
-                    }
+                    goto found;
                 }
-                goto not_found;
             }
-            else if (type != CLOSURE_UNBOUND_LAMBDA)
-            {
-                lambda_t *l;
 
-                l = fun->u.lambda;
-                if (type != CLOSURE_LFUN)
-                type = CLOSURE_UNBOUND_LAMBDA;
-                for (copp = &call_list; NULL != (cop = *copp); copp = &cop->next)
-                {
-                    delay += cop->delta;
-                    if (cop->fun.is_lambda
-                     && (   cop->fun.function.lambda.u.lambda == l
-                         || (   cop->fun.function.lambda.x.closure_type == type
-                             && cop->fun.function.lambda.u.lambda->ob == l->ob
-                             && cop->fun.function.lambda.u.lambda->function.lfun.index
-                                == l->function.lfun.index)
-                        )
-                       )
-                    {
-                        goto found;
-                    }
-                }
-                goto not_found;
-            }
-            else if (type == CLOSURE_UNBOUND_LAMBDA)
-            {
-not_found:
-                free_svalue(fun);
-                put_number(fun, -1);
-                return;
+            /* Not found */
+            free_svalue(fun);
+            put_number(fun, -1);
+            return;
 found:
-                free_svalue(fun);
-                if (do_free_call)
-                {
-                    if (cop->next)
-                        cop->next->delta += cop->delta;
-                    *copp = cop->next;
-                    free_call(cop);
-                    num_callouts--;
-                }
-                /* It is possible to have delay < 0 if we are
-                 * called from inside call_out() .
-                 */
-                if (delay < 0)
-                    delay = 0;
-                put_number(fun, delay);
-                return;
+            free_svalue(fun);
+            if (do_free_call)
+            {
+                if (cop->next)
+                    cop->next->delta += cop->delta;
+                *copp = cop->next;
+                free_call(cop);
+                num_callouts--;
             }
+            /* It is possible to have delay < 0 if we are
+             * called from inside call_out() .
+             */
+            if (delay < 0)
+                delay = 0;
+            put_number(fun, delay);
+            return;
         }
-        bad_efun_arg(1, -1, fun);
+        fatal("find_call_out() got %s, expected string/closure.\n"
+             , typename(fun->type));
         /* NOTREACHED */
     }
 
 
     /* Find callout by object/name */
 
-    if (fun->x.string_type == STRING_SHARED) {
-        fun_name = fun->u.string;
-    }
-    else
-    {
-        fun_name = make_shared_string(fun->u.string);
-        if (fun->x.string_type == STRING_MALLOC)
-            xfree(fun->u.string);
-    }
+    fun_name = find_tabled(fun->u.str);
 
+    if (fun_name != NULL)
     for (copp = &call_list; NULL != (cop = *copp); copp = &cop->next)
     {
         delay += cop->delta;
@@ -553,7 +525,6 @@ found:
          && cop->fun.function.named.name == fun_name
          && !cop->fun.is_lambda)
         {
-            deref_string(fun_name);
             if (do_free_call)
             {
                 if (cop->next)
@@ -564,11 +535,14 @@ found:
             }
             if (delay < 0)
                 delay = 0;
+            free_svalue(fun);
             put_number(fun, delay);
             return;
         }
-    }
-    free_string(fun_name);
+    } /* if()for() */
+
+    /* Not found */
+    free_svalue(fun);
     put_number(fun, -1);
 } /* find_call_out() */
 
@@ -721,12 +695,12 @@ count_ref_from_call_outs (void)
             }
         }
     }
-}
+} /* count_ref_from_call_outs() */
 
 #endif /* GC_SUPPORT */
 
 /*-------------------------------------------------------------------------*/
-vector_t *
+static vector_t *
 get_all_call_outs (void)
 
 /* Construct an array of all pending call_outs (whose object is not
@@ -774,9 +748,9 @@ get_all_call_outs (void)
 
         if (cop->fun.is_lambda)
         {
-            if (cop->fun.function.lambda.x.closure_type == CLOSURE_ALIEN_LFUN)
+            if (cop->fun.function.lambda.x.closure_type == CLOSURE_LFUN)
                 put_ref_object( vv->item
-                              , cop->fun.function.lambda.u.lambda->function.alien.ob
+                              , cop->fun.function.lambda.u.lambda->function.lfun.ob
                               , "get_all_call_outs");
             else
                 put_ref_object(vv->item, ob, "get_all_call_outs");
@@ -813,5 +787,75 @@ get_all_call_outs (void)
     return v;
 } /* get_all_call_outs() */
 
+/*-------------------------------------------------------------------------*/
+svalue_t *
+f_call_out_info (svalue_t *sp)
+
+/* EFUN call_out_info()
+ *
+ *     mixed *call_out_info(void)
+ *
+ * Get information about all pending call outs. The result is an
+ * array in which every entry is itself an array describing one
+ * call_out.
+ *
+ * The efun causes the the privilege violation ("call_out_info",
+ * this_object()). If it is not satisfied, the result will be
+ * the empty array.
+ */
+
+{
+    if (privilege_violation(STR_CALL_OUT_INFO, &const0, sp))
+    {
+        push_array(sp, get_all_call_outs());
+    }
+    else
+    {
+        push_ref_array(sp, &null_vector);
+    }
+
+    return sp;
+} /* f_call_out_info() */
+
+/*-------------------------------------------------------------------------*/
+svalue_t *
+f_find_call_out (svalue_t *sp)
+
+/* EFUN find_call_out()
+ *
+ *   int find_call_out(string func)
+ *   int find_call_out(closure func)
+ *
+ * Find the first call-out due to be executed for function func
+ * in the current object, and return the time left. If no call-out
+ * is found return -1.
+ */
+
+{
+    find_call_out(current_object, sp, MY_FALSE);
+    return sp;
+} /* f_find_call_out() */
+
+/*-------------------------------------------------------------------------*/
+svalue_t *
+f_remove_call_out (svalue_t *sp)
+
+/* EFUN remove_call_out()
+ *
+ *   int remove_call_out(string fun)
+ *   int remove_call_out(closure fun)
+ *
+ * Remove next pending call-out for function fun in this object.
+ * The time left is returned.
+ *
+ * -1 is returned if there were no call-outs pending to this
+ * function.
+ */
+
+{
+    find_call_out(current_object, sp, MY_TRUE);
+    return sp;
+} /* f_remove_call_out() */
+
 /***************************************************************************/
 
