@@ -2,7 +2,7 @@
  * Specialised Memory Allocators.
  *
  *---------------------------------------------------------------------------
- * Memory buffers provide memory for functions which repeatedly allocated
+ * Memory buffers provide memory for functions which repeatedly allocate
  * large chunks of temporary memory (restore_object() for example). Instead
  * of deallocating the memory after its use, the memory buffer keeps it
  * around for the next time. This way any interim fragmentation of the free
@@ -13,14 +13,28 @@
  *---------------------------------------------------------------------------
  * Purpose of memory pools is to provide fast allocation methods for
  * certain allocation patterns. They are most useful for scratchpad
- * purposes.
+ * purposes where a largish number of small allocations need to be
+ * deallocated at once.
  *
  * Mempools: allocation of objects with identical time of death.
  *           Attempts to deallocate single objects have no effect.
  *
  * Lifopools: allocation/deallocation of objects follows (more than less)
  *            a lifo pattern.
- *---------------------------------------------------------------------------
+ *
+ * TODO: A small-block pool, to manage lots of small blocks of equal size
+ * TODO:: without the overhead of smalloc. Initialized with the block size,
+ * TODO:: the number of initial blocks, and the number of blocks each
+ * TODO:: the pool has to grow. These blocks can be freed and are then
+ * TODO:: kept in a freelist for later reuse. To help compacting the pool
+ * TODO:: a method alloc_before(void *) would allocate a free small block
+ * TODO:: before the given one - if there is no such free block, the
+ * TODO:: method would return NULL. With this method a user of the
+ * TODO:: can compact its data at the front of the pool blocks.
+ * TODO:: The last unused pool block is freed only if either another small
+ * TODO:: block is freed, or the method flush_free_pool_blocks() (or so)
+ * TODO:: is called.
+ *
  * Memory pools can be made dependant from other pools. This means that
  * whenever the 'superior' pool is reset or deleted, all pools depending
  * on this superior one are also reset or deleted. This is a
@@ -57,6 +71,7 @@
 #include "driver.h"
 
 #include <assert.h>
+#include <stddef.h>
 #include <stdlib.h>
 #include <sys/types.h>
 
@@ -251,12 +266,12 @@ mb_status (strbuf_t * sbuf, Bool verbose)
     {
         strbuf_add(sbuf, "\nMemory Buffers:\n");
         strbuf_add(sbuf,   "---------------\n");
-        strbuf_addf(sbuf, "File data:    %8lu\n", membuffers[mbFile].size);
-        strbuf_addf(sbuf, "Swap buffer:  %8lu\n", membuffers[mbSwap].size);
+        strbuf_addf(sbuf, "File data:    %8zu\n", membuffers[mbFile].size);
+        strbuf_addf(sbuf, "Swap buffer:  %8zu\n", membuffers[mbSwap].size);
     }
     else
     {
-        strbuf_addf(sbuf, "Memory buffers:\t\t\t\t %9lu\n", res);
+        strbuf_addf(sbuf, "Memory buffers:\t\t\t\t %9zu\n", res);
     }
 
     return res;
@@ -346,7 +361,7 @@ struct memblock_s {
     } u;
 };
 
-#define MEMBLOCK_LIMIT (256)
+#define MEMBLOCK_LIMIT (128)
   /* Maximum size for the userspace of a memory block.
    * Using blocks larger than this is likely to run into fragmentation
    * of the large block heap.
@@ -446,7 +461,7 @@ size_lifopool (size_t elemsize)
 {
     return size_pool(elemsize, SIZEOF_LIFO_T);
 } /* size_lifopool() */
- 
+
 /*-------------------------------------------------------------------------*/
 static INLINE Mempool
 new_pool (size_t iSize, pooltype_t type)
@@ -466,7 +481,7 @@ new_pool (size_t iSize, pooltype_t type)
       /* If this assert fails, sizeof(union align) is not a power of 2 */
 
     /* Round iSize up to the next integral of sizeof(union align) */
-    iSize = ROUND(iSize + (type == LIFOPOOL ? SIZEOF_LIFO_T : 0));
+    iSize = ROUND(iSize);
 
     pPool = xalloc(sizeof(*pPool));
     if (!pPool)
@@ -483,19 +498,6 @@ new_pool (size_t iSize, pooltype_t type)
     pBlock->length = sizeof(*pBlock) - sizeof(pBlock->u) + iSize;
     pBlock->pMark = pBlock->u.data + iSize;
 
-    /* For lifopools, add a sentinel (pseudo-used block) at the end
-     * of the arena.
-     */
-    if (LIFOPOOL == type)
-    {
-        lifo_t *p = (lifo_t *)(pBlock->pMark - SIZEOF_LIFO_T);
-
-        p->length = 1;
-        p->pBlock = pBlock;
-
-        /* Update the pMark pointer */
-        pBlock->pMark = (char *)p;
-    }
     /* Setup the pool */
     pPool->type = type;
     pPool->iAllocSize = iSize;
@@ -519,7 +521,7 @@ new_mempool (size_t iSize)
 
 {
     return new_pool(iSize, MEMPOOL);
-}
+} /* new_mempool() */
 
 /*-------------------------------------------------------------------------*/
 Mempool
@@ -532,8 +534,26 @@ new_lifopool (size_t iSize)
  */
 
 {
-    return new_pool(iSize, LIFOPOOL);
-}
+    Mempool pPool;
+
+    iSize += SIZEOF_LIFO_T; /* Include space for the sentinel block */
+
+    pPool = new_pool(iSize, LIFOPOOL);
+    if (pPool)
+    {
+        /* Add a sentinel (pseudo-used block) at the end of the arena.
+         */
+        struct memblock_s * pBlock = pPool->pBlocks;
+        lifo_t *p = (lifo_t *)(pBlock->pMark - SIZEOF_LIFO_T);
+
+        p->length = 1;
+        p->pBlock = pBlock;
+
+        /* Update the pMark pointer */
+        pBlock->pMark = (char *)p;
+    }
+    return pPool;
+} /* new_lifopool() */
 
 /*-------------------------------------------------------------------------*/
 void
@@ -595,7 +615,7 @@ alloc_from_pool (Mempool pPool, size_t iSize)
      */
 
     pBlock = pPool->pBlocks;
-    if (iSize > pBlock->pMark - pBlock->u.data)
+    if ((ptrdiff_t)iSize > pBlock->pMark - pBlock->u.data)
     {
         pBlock = xalloc(  sizeof(*pBlock)-sizeof(pBlock->u)
                         + pPool->iAllocSize);
@@ -672,7 +692,7 @@ alloc_from_lifo (Mempool pPool, size_t iSize)
      * memory left, a new one is allocated.
      */
     pBlock = pPool->pBlocks;
-    if (iSize > pBlock->pMark - pBlock->u.data)
+    if ((ptrdiff_t)iSize > pBlock->pMark - pBlock->u.data)
     {
         /* If there are blocks in the freelist, use those first;
          * otherwise allocate a new one.
@@ -797,7 +817,7 @@ mempool_free (Mempool pPool, void * adr)
     {
         while (pBlock->pNext != NULL
             &&    pBlock->pMark - (char*)&(pBlock->u)
-               >= pPool->iAllocSize - SIZEOF_LIFO_T)
+               >= (ptrdiff_t)(pPool->iAllocSize - SIZEOF_LIFO_T))
         {
             pPool->pBlocks = pBlock->pNext;
             pBlock->pNext = pPool->pFree;
