@@ -18,7 +18,7 @@
 extern void * zlib_alloc (void *opaque, unsigned int items, unsigned int size);
 extern void zlib_free (void *opaque, void *address);
 extern Bool start_compress (interactive_t * ip, unsigned char telopt);
-extern Bool end_compress (interactive_t * ip);
+extern Bool end_compress (interactive_t * ip, Bool force);
 
 extern svalue_t * f_start_mccp_compress (svalue_t * sp);
 extern svalue_t * f_end_mccp_compress (svalue_t * sp);
