@@ -6,13 +6,13 @@
  * This file holds the efuns interfacing with mySQL. See the file INSTALL
  * or doc/concepts/mysql for setup instructions.
  *
- *    tefun: db_connect()
- *    tefun: db_close()
- *    tefun: db_exec()
- *    tefun: db_fetch()
- *    tefun: db_affected_rows()
- *    tefun: db_conv_string()
- *    tefun: db_handles()
+ *   efun: db_connect()
+ *   efun: db_close()
+ *   efun: db_exec()
+ *   efun: db_fetch()
+ *   efun: db_affected_rows()
+ *   efun: db_conv_string()
+ *   efun: db_handles()
  *
  *---------------------------------------------------------------------------
  */
@@ -26,6 +26,7 @@
 #include "my-alloca.h"
 #include <stddef.h>
 #include <string.h>
+#include <stdio.h>
 
 #include <mysql.h>
 #include <errmsg.h>
@@ -35,51 +36,14 @@
 
 #include "array.h"
 #include "interpret.h"
+#include "instrs.h"
 #include "main.h"
+#include "mstrings.h"
 #include "simulate.h"
+#include "stdstrings.h"
 #include "svalue.h"
 #include "xalloc.h"
 
-/* Macros */
-
-/* Typetests for xefuns/tefuns */
-
-#define TYPE_TEST1(arg1,type1) \
-  if ((arg1)->type != type1) \
-      bad_xefun_arg(1, sp);
-
-#define TYPE_TEST2(arg1,type1) \
-  if ((arg1)->type != type1) \
-      bad_xefun_arg(2, sp);
-
-#define TYPE_TEST3(arg1,type1) \
-  if ((arg1)->type != type1) \
-      bad_xefun_arg(3, sp);
-
-/* Typetests for vararg xefuns/vefuns */
-
-#define TYPE_TESTV1(arg1,type1) \
-  if ((arg1)->type != type1) \
-      bad_xefun_vararg(1, sp);
-
-#define TYPE_TESTV2(arg1,type1) \
-  if ((arg1)->type != type1) \
-      bad_xefun_vararg(2, sp);
-
-#define TYPE_TESTV3(arg1,type1) \
-  if ((arg1)->type != type1) \
-      bad_xefun_vararg(3, sp);
-
-/* Typetests for efuns */
-
-#define E_TYPE_TESTV1(arg1,type1) \
-  if ((arg1)->type != type1) \
-      bad_efun_arg(1, -2, sp);
-
-#define E_TYPE_TESTV2(arg1,type1) \
-  if ((arg1)->type != type1) \
-      bad_efun_arg(2, -2, sp);
-
 /*-------------------------------------------------------------------------*/
 
 typedef struct db_dat_s db_dat_t;
@@ -158,16 +122,20 @@ Bool check_privilege (const char * efun_name, Bool raise_error, svalue_t * sp)
  */
 
 {
-    svalue_t tmp;
+    Bool rc;
 
-    inter_sp = sp;
-    put_volatile_string(&tmp, (char *)efun_name);
-    if (privilege_violation3("mysql", &tmp, sp))
+    inter_sp = sp+1;
+    put_c_string(inter_sp, efun_name);
+    rc = privilege_violation(STR_MYSQL, inter_sp, inter_sp);
+    free_svalue(inter_sp);
+    inter_sp--;
+
+    if (rc)
         return MY_TRUE;
 
     if (raise_error)
     {
-        error("%s(): Privilege violation.\n", efun_name);
+        errorf("%s(): Privilege violation.\n", efun_name);
         /* NOTREACHED */
     }
 
@@ -191,7 +159,7 @@ allocate_new_dat(void)
         my_dat = pxalloc(sizeof(*my_dat));
        if ( !my_dat )
        {
-           error("Out of memory.\n");
+           errorf("Out of memory.\n");
            /* NOTREACHED */
            return NULL;
        }
@@ -208,7 +176,7 @@ allocate_new_dat(void)
     tmp = my_dat->prev = pxalloc(sizeof(db_dat_t));
     if ( !tmp )
     {
-       error("Out of memory.\n");
+       errorf("Out of memory.\n");
        return NULL;
     }
     tmp->next = my_dat; /* Put the new handle to the beginning */
@@ -346,19 +314,18 @@ raise_db_error (db_dat_t *dat)
     const char *tmp;
     char *err_string;
 
-    if ( !dat )
+    if ( !dat || !(tmp = mysql_error(dat->mysql_dat))[0] )
     {
-        error( "An unknown error occured during the current database-"
+        errorf( "An unknown error occured during the current database-"
                "operation\n");
         /* NOTREACHED */
        abort();
     }
-    tmp = mysql_error(dat->mysql_dat);
     err_string = alloca(strlen(tmp) + 2);
     strcpy(err_string, tmp);
     strcat(err_string, "\n");
     remove_dat(dat);
-    error(err_string);
+    errorf(err_string);
     /* NOTREACHED */
     abort();
 } /* raise_db_error() */
@@ -367,7 +334,7 @@ raise_db_error (db_dat_t *dat)
 svalue_t *
 f_db_affected_rows (svalue_t *sp)
 
-/* TEFUN db_affected_rows()
+/* EFUN db_affected_rows()
  *
  *   int db_affected_rows(int handle)
  *
@@ -381,11 +348,10 @@ f_db_affected_rows (svalue_t *sp)
     int          rows;
     unsigned int handle;
 
-    TYPE_TEST1(sp, T_NUMBER)
     check_privilege(instrs[F_DB_AFFECTED_ROWS].name, MY_TRUE, sp);
     handle = (unsigned int)sp->u.number;
     if ( !(dat = find_dat_by_handle(handle)) )
-        error("Illegal handle for database.\n");
+        errorf("Illegal handle for database.\n");
     rows = mysql_affected_rows(dat->mysql_dat);
     free_svalue(sp); /* Well, it's just a number */
     put_number(sp, rows);
@@ -396,7 +362,7 @@ f_db_affected_rows (svalue_t *sp)
 svalue_t *
 f_db_conv_string (svalue_t *sp)
 
-/* TEFUN db_conv_string()
+/* EFUN db_conv_string()
  *
  *   string db_conv_string(string str)
  *
@@ -406,25 +372,26 @@ f_db_conv_string (svalue_t *sp)
  */
 
 {
-    char *s;
+    string_t *s;
     char *buff;
 
-    TYPE_TEST1(sp, T_STRING)
-    s = sp->u.string;
-    buff = alloca(strlen(s)*2 +1);
+    s = sp->u.str;
+    buff = xalloc_with_error_handler(mstrsize(s)*2 +1);
+    // top of stack / inter_sp is now one larger than sp!
     if ( !buff )
     {
-        error("Out of memory.\n");
+        errorf("Out of memory (%zu bytes) in db_conv_string().\n",
+               mstrsize(s)*2 + 1);
         /* NOTREACHED */
         return sp;
     }
-    mysql_escape_string(buff, s, strlen(s));
-
-    /* Re-allocate the escaped string to its proper length */
-    s = string_copy(buff);
-
+    mysql_escape_string(buff, get_txt(s), strlen(get_txt(s)) );
+    
     free_string_svalue(sp);
-    put_malloced_string(sp, s);
+    put_c_string(sp, buff);
+    // the error handler is one above sp on the stack.
+    pop_stack();
+    // sp == inter_sp again.
     return sp;
 } /* f_db_conv_string() */
 
@@ -432,7 +399,7 @@ f_db_conv_string (svalue_t *sp)
 svalue_t *
 f_db_close (svalue_t *sp)
 
-/* TEFUN db_close()
+/* EFUN db_close()
  *
  *   int db_close(int handle)
  *
@@ -444,12 +411,11 @@ f_db_close (svalue_t *sp)
     p_int     handle;
     db_dat_t *dat;
 
-    TYPE_TEST1(sp, T_NUMBER)
     check_privilege(instrs[F_DB_CLOSE].name, MY_TRUE, sp);
     handle = sp->u.number;
     if ( !(dat = find_dat_by_handle((unsigned int)handle)) )
     {
-        error("Illegal handle for database.\n");
+        errorf("Illegal handle for database.\n");
         /* NOTREACHED */
         return sp;
     }
@@ -461,9 +427,9 @@ f_db_close (svalue_t *sp)
 
 /*-------------------------------------------------------------------------*/
 svalue_t *
-f_db_connect (svalue_t *sp, int num_args)
+v_db_connect (svalue_t *sp, int num_args)
 
-/* TEFUN db_connect()
+/* EFUN db_connect()
  *
  *   int db_connect(string database, void|string user, void|string password)
  *
@@ -476,46 +442,34 @@ f_db_connect (svalue_t *sp, int num_args)
  */
 
 {
-    char     *database, *user, *password;
+    string_t *database, *user, *password;
     p_int     sock;
     db_dat_t *tmp;
 
     check_privilege(instrs[F_DB_CONNECT].name, MY_TRUE, sp);
-
     switch(num_args)
     {
-      case 3:
-        TYPE_TESTV1(sp-2, T_STRING);
-        TYPE_TESTV2(sp-1, T_STRING);
-        TYPE_TESTV3(sp, T_STRING);
-        database = sp[-2].u.string;
-        user = sp[-1].u.string;
-        password = sp->u.string;
+    case 3:
+        database = sp[-2].u.str;
+        user = sp[-1].u.str;
+        password = sp->u.str;
         break;
-      case 2:
-        TYPE_TESTV1(sp-1, T_STRING);
-        TYPE_TESTV2(sp,   T_STRING);
-        database = sp[-1].u.string;
-        user = sp->u.string;
-        password = 0;
+    case 2:
+        database = sp[-1].u.str;
+        user = sp->u.str;
+        password = NULL;
         break;
-      case 1:
-        TYPE_TESTV1(sp, T_STRING);
-        database = sp->u.string;
-        user     = 0;
-        password = 0;
+    default:
+        database = sp->u.str;
+        user     = NULL;
+        password = NULL;
         break;
-      default:
-        /* Compiler makes sure that we get at least one arg */
-        error("Too many arguments to db_connect()\n");
-        /* NOTREACHED */
-        return NULL;
     }
 
     tmp = allocate_new_dat();
     if ( !tmp )
     {
-        error("Out of memory.\n");
+        errorf("Out of memory.\n");
         /* NOTREACHED */
         return NULL;
     }
@@ -524,7 +478,7 @@ f_db_connect (svalue_t *sp, int num_args)
     if ( !tmp->mysql_dat )
     {
         remove_dat(tmp);
-        error("Out of memory.\n");
+        errorf("Out of memory.\n");
         /* NOTREACHED */
         return NULL;
     }
@@ -533,14 +487,37 @@ f_db_connect (svalue_t *sp, int num_args)
      * I wouldn't dare to implement synchronous DB-access via
      * TCP (that's something for ERQ wizards :-).
      */
-    if ( !mysql_real_connect(tmp->mysql_dat, "localhost", user,
-                             password, database, 0, 0, 0))
+    if ( !mysql_real_connect( tmp->mysql_dat, "localhost"
+                            , user ? get_txt(user) : NULL
+                            , password ? get_txt(password) : NULL
+                            , get_txt(database)
+                            , 0, 0, 0))
     {
         raise_db_error(tmp);
         /* NOTREACHED */
         return sp;
     }
 
+    /* MySQL before 5.0.3 used to reconnect to servers automatically when
+     * the connection was lost. For later versions, we have to enable this
+     * this feature ourselves.
+     *
+     * Note that while this is convenient, any session state like SET ...
+     * commands or temporary tables will be lost silently on reconnects. If
+     * this becomes a problem for anybody we need to rethink this default.
+     */
+#if MYSQL_VERSION_ID >= 50003
+    {
+        my_bool my_true = MY_TRUE;
+        if (mysql_options(tmp->mysql_dat, MYSQL_OPT_RECONNECT, &my_true))
+        {
+            raise_db_error(tmp);
+            /* NOTREACHED */
+            return sp;
+        }
+    }
+#endif
+
     switch (num_args)
     {
       case 3:
@@ -556,13 +533,13 @@ f_db_connect (svalue_t *sp, int num_args)
     sock = (signed)tmp->handle;
     put_number(sp, sock);
     return sp;
-} /* f_db_connect() */
+} /* v_db_connect() */
 
 /*-------------------------------------------------------------------------*/
 svalue_t *
 f_db_error (svalue_t *sp)
 
-/* TEFUN db_error()
+/* EFUN db_error()
  *
  *   string db_error(int handle)
  *
@@ -576,15 +553,12 @@ f_db_error (svalue_t *sp)
     unsigned int  handle;
     const char   *errmsg;
 
-    TYPE_TEST1(sp, T_NUMBER)
-
     check_privilege(instrs[F_DB_ERROR].name, MY_TRUE, sp);
-
     handle = (unsigned int)sp->u.number;
 
     if ( !(dat = find_dat_by_handle(handle)) )
     {
-        error("Illegal handle for database.\n");
+        errorf("Illegal handle for database.\n");
         /* NOTREACHED */
         return sp;
     }
@@ -598,11 +572,8 @@ f_db_error (svalue_t *sp)
     }
     else
     {
-        char *s;
         free_svalue(sp);
-        s = xalloc(strlen(errmsg)+1);
-        strcpy(s, errmsg);
-        put_malloced_string(sp, s);
+        put_c_string(sp, errmsg);
     }
 
     return sp;
@@ -612,7 +583,7 @@ f_db_error (svalue_t *sp)
 svalue_t *
 f_db_exec (svalue_t *sp)
 
-/* TEFUN db_exec()
+/* EFUN db_exec()
  *
  *   int db_exec(int handle, string statement)
  *
@@ -622,22 +593,18 @@ f_db_exec (svalue_t *sp)
  */
 
 {
-    char         *s;
+    string_t     *s;
     db_dat_t     *dat;
-    unsigned int  handle;
     unsigned int  err_no;
-
-    TYPE_TEST1(sp-1, T_NUMBER)
-    TYPE_TEST2(sp, T_STRING)
-
-    check_privilege(instrs[F_DB_EXEC].name, MY_TRUE, sp);
+    unsigned int  handle;
 
     handle = (unsigned int)sp[-1].u.number;
-    s = sp->u.string;
+    s = sp->u.str;
 
+    check_privilege(instrs[F_DB_EXEC].name, MY_TRUE, sp);
     if ( !(dat = find_dat_by_handle(handle)) )
     {
-        error("Illegal handle for database.\n");
+        errorf("Illegal handle for database.\n");
         /* NOTREACHED */
         return sp;
     }
@@ -648,7 +615,7 @@ f_db_exec (svalue_t *sp)
         dat->mysql_result = NULL;
     }
 
-    if ( mysql_query(dat->mysql_dat, s) )
+    if ( mysql_query(dat->mysql_dat, get_txt(s)) )
     {
         /* either a REAL error occured or just an error in the SQL-statement
          */
@@ -700,7 +667,7 @@ f_db_exec (svalue_t *sp)
 svalue_t *
 f_db_fetch (svalue_t *sp)
 
-/* TEFUN db_fetch()
+/* EFUN db_fetch()
  *
  *   mixed db_fetch(int handle)
  *
@@ -715,13 +682,11 @@ f_db_fetch (svalue_t *sp)
     int           num_cols, i;
     unsigned int  handle;
 
-    TYPE_TEST1(sp, T_NUMBER)
     check_privilege(instrs[F_DB_FETCH].name, MY_TRUE, sp);
-
     handle = (unsigned int)sp->u.number;
     if ( !(dat = find_dat_by_handle(handle)) )
     {
-        error("Illegal handle for database.\n");
+        errorf("Illegal handle for database.\n");
         /* NOTREACHED */
         return sp;
     }
@@ -752,14 +717,14 @@ f_db_fetch (svalue_t *sp)
     v = allocate_array(num_cols);
     if (!v)
     {
-        error("Out of memory.\n");
+        errorf("Out of memory.\n");
         /* NOTREACHED */
         return sp;
     }
 
     for (i = 0; i < num_cols; i++)
         if (dat->mysql_row[i])
-            put_malloced_string(v->item+i, string_copy(dat->mysql_row[i]));
+            put_c_string(v->item+i, dat->mysql_row[i]);
         /* else return 0 for that entry */
 
     free_svalue(sp); /* It's a number */
@@ -771,7 +736,7 @@ f_db_fetch (svalue_t *sp)
 svalue_t *
 f_db_handles (svalue_t *sp)
 
-/* TEFUN db_handles()
+/* EFUN db_handles()
  *
  *   int *db_handles()
  *
@@ -790,7 +755,6 @@ f_db_handles (svalue_t *sp)
     vector_t *v;
 
     check_privilege(instrs[F_DB_HANDLES].name, MY_TRUE, sp);
-
     tmp = my_dat;
 
     /* Maybe there's no open connection yet/anymore */
@@ -799,7 +763,7 @@ f_db_handles (svalue_t *sp)
         v = allocate_array(0);
         if (!v)
         {
-            error("Out of memory.\n");
+            errorf("Out of memory.\n");
             /* NOTREACHED */
             return sp;
         }
@@ -818,7 +782,7 @@ f_db_handles (svalue_t *sp)
     v = allocate_array(elems);
     if (!v)
     {
-        error("Out of memory.\n");
+        errorf("Out of memory.\n");
         /* NOTREACHED */
         return sp;
     }
@@ -854,11 +818,10 @@ f_db_insert_id (svalue_t *sp)
     my_ulonglong  insertid;
     unsigned int  handle;
 
-    TYPE_TEST1(sp, T_NUMBER)
     check_privilege(instrs[F_DB_INSERT_ID].name, MY_TRUE, sp);
     handle = (unsigned int)sp->u.number;
     if ( !(dat = find_dat_by_handle(handle)) )
-        error("Illegal handle for database.\n");
+        errorf("Illegal handle for database.\n");
     insertid = mysql_insert_id(dat->mysql_dat);
     free_svalue(sp); /* Well, it's just a number */
     put_number(sp, insertid);
@@ -885,12 +848,11 @@ f_db_coldefs (svalue_t *sp)
     unsigned int  handle;
     MYSQL_FIELD  *fields;
 
-    TYPE_TEST1(sp, T_NUMBER)
     check_privilege(instrs[F_DB_COLDEFS].name, MY_TRUE, sp);
     handle = (unsigned int)sp->u.number;
     if ( !(dat = find_dat_by_handle(handle)) )
     {
-        error("Illegal handle for database.\n");
+        errorf("Illegal handle for database.\n");
         /* NOTREACHED */
         return sp;
     }
@@ -906,7 +868,7 @@ f_db_coldefs (svalue_t *sp)
     v = allocate_array(num_fields);
     if (!v)
     {
-        error("Out of memory for result array (%d elements).\n", num_fields);
+        errorf("Out of memory for result array (%d elements).\n", num_fields);
         /* NOTREACHED */
         return sp;
     }
@@ -915,7 +877,7 @@ f_db_coldefs (svalue_t *sp)
 
     for (i = 0; i < num_fields; i++)
     {
-        put_malloced_string(v->item+i, string_copy(fields[i].name) );
+        put_c_string(v->item+i, fields[i].name);
     }
 
     free_svalue(sp); /* It's a number */
