@@ -21,9 +21,9 @@
  * If such a discarded simul-efun is re-implemented by a new simul-efun object,
  * the old table entry is reactivated.
  *
- * The first 256 simul-efuns of this table are mirrored in a second table
- * (simul_efun_table) from where they are called by index with the special
- * SIMUL_EFUN instruction.
+ * The first SEFUN_TABLE_SIZE simul-efuns of this table are mirrored in a
+ * second table (simul_efun_table) from where they are called by index with
+ * the special SIMUL_EFUN instruction.
  *---------------------------------------------------------------------------
  */
 
@@ -38,14 +38,14 @@
 #include "array.h"
 #include "exec.h"
 #include "gcollect.h"
-#include "instrs.h"
 #include "interpret.h"
 #include "lex.h"
+#include "mstrings.h"
 #include "object.h"
 #include "ptrtable.h"
+#include "prolang.h" /* is_undef_function() */
 #include "simulate.h"
 #include "stdstrings.h"
-#include "stralloc.h"
 #include "svalue.h"
 #include "swap.h"
 #include "xalloc.h"
@@ -74,7 +74,7 @@ vector_t *simul_efun_vector  = NULL;
 
 #define SIZE_SEFUN_TABLE (sizeof simul_efun_table / sizeof simul_efun_table[0])
 
-simul_efun_table_t simul_efun_table[256];
+simul_efun_table_t simul_efun_table[SEFUN_TABLE_SIZE];
   /* The table holding the information for all simul-efuns which
    * can be called directly with the SIMUL_EFUN instruction.
    * A .funstart of NULL marks unused/discarded entries.
@@ -88,7 +88,7 @@ static int total_simul_efun  = 0;
   /* Allocated size of simul_efunp.
    */
 
-static char *simul_efun_file_name = NULL;
+static string_t *simul_efun_file_name = NULL;
   /* The shared name of the primary simul_efun object
    */
 
@@ -171,15 +171,17 @@ invalidate_simul_efuns (void)
 } /* invalidate_simul_efuns() */
 
 /*-------------------------------------------------------------------------*/
-object_t *
-get_simul_efun_object (Bool require)
+Bool
+assert_simul_efun_object (void)
 
 /* (Re)load the simul_efun object and extract all information we need.
- * Result is a copy of the simul_efun_object pointer, or NULL on failure.
+ * Result is TRUE if either the simul_efun object could be loaded, or if
+ * master::get_simul_efun() did not return a string/string vector to
+ * name the simul efun object. The result is FALSE if master::get_simul_efun()
+ * specified a simul efun object, which couldn't be found.
  *
- * If <require> is TRUE and the master lfun get_simul_efun() returns
- * the name of an object, but this object doesn't exist, then the function
- * terminates the driver.
+ * In other words: after calling assert_simul_efun_object(), the caller
+ * still has to check if simul_efun_object is NULL.
  *
  * At the time of call, simul_efun_object must be NULL.
  */
@@ -189,7 +191,7 @@ get_simul_efun_object (Bool require)
     object_t           *ob;
     program_t          *progp;
     CBool              *visible; /* Flag for every function: visible or not */
-    char               *name;
+    string_t           *name;
     int                 i, j, num_fun;
 
     invalidate_simul_efuns(); /* Invalidate the simul_efun information */
@@ -198,10 +200,22 @@ get_simul_efun_object (Bool require)
 
     /* Get the name(s) of the simul_efun  object. */
     svp = apply_master(STR_GET_SEFUN, 0);
+
+    /* If a simul_efun_object appears during the GET_SEFUN call, it
+     * might have been due to a recursive get_simul_efun() call which may
+     * have gotten an old backup copy. This can lead to hard-to-debug
+     * variable and function definition inconsistencies.
+     */
+    if (simul_efun_object)
+    {
+        printf("%s simul_efun object appeared while asking for it.\n", time_stamp());
+        return MY_TRUE;
+    }
+
     if (svp == NULL)
     {
         printf("%s No simul_efun\n", time_stamp());
-        return NULL;
+        return MY_TRUE;
     }
 
     if (svp->type == T_POINTER)
@@ -215,40 +229,36 @@ get_simul_efun_object (Bool require)
     if (svp->type != T_STRING)
     {
         printf("%s No simul_efun\n", time_stamp());
-        return NULL;
+        return MY_TRUE;
     }
 
     /* Make the (primary) simul_efun name */
-    name = svp->u.string;
-    while (*name == '/')
-        name++;
+    name = del_slash(svp->u.str);
     if (simul_efun_file_name)
-        free_string(simul_efun_file_name);
-    simul_efun_file_name = make_shared_string(name);
+        free_mstring(simul_efun_file_name);
+    simul_efun_file_name = make_tabled(name);
 
     /* Get the object and load the program */
     ob = find_object(simul_efun_file_name);
     if (ob == NULL)
     {
         fprintf(stderr, "%s The simul_efun file %s was not loaded.\n"
-               , time_stamp(), simul_efun_file_name);
+               , time_stamp(), get_txt(simul_efun_file_name));
         fprintf(stderr, "%s The function get_simul_efun() in the master must load it.\n"
                , time_stamp());
-        if (require)
-            exit(1);
-        return NULL;
+        return MY_FALSE;
     }
     if (O_PROG_SWAPPED(ob) && load_ob_from_swap(ob) < 0)
     {
         fprintf(stderr, "%s Out of memory (unswap object '%s') ==> "
-                        "No simul_efun\n", time_stamp(), ob->name);
-        return NULL;
+                        "No simul_efun\n", time_stamp(), get_txt(ob->name));
+        return MY_TRUE;
     }
     reference_prog( (simul_efun_program = ob->prog), "get_simul_efun");
 
     num_fun = ob->prog->num_function_names;
     if (num_fun == 0)
-        return NULL;
+        return MY_TRUE;
     if (!simul_efunp)
     {
         simul_efunp = xalloc(sizeof (function_t) * num_fun);
@@ -313,34 +323,41 @@ get_simul_efun_object (Bool require)
         funstart = inherit_progp->program + (flags2 & FUNSTART_MASK);
 
         /* Don't stumble over undefined functions */
-        if (GET_CODE(FUNCTION_CODE(funstart)) == F_ESCAPE
-         && GET_CODE(FUNCTION_CODE(funstart)+1) == F_UNDEF - 0x100)
+        if (is_undef_function(funstart))
         {
             flags |= NAME_UNDEFINED;
         }
 
-        /* If the function is indeed visible, get its information */
+        /* If the function is __INIT, pretend it's a private function */
         if ( !(flags & (TYPE_MOD_STATIC|TYPE_MOD_PRIVATE|NAME_UNDEFINED)) )
         {
-            char *function_name;
-            ident_t *p;
-            unsigned char type, num_arg, num_locals;
+            string_t *function_name;
 
             memcpy(  &function_name, FUNCTION_NAMEP(funstart)
                    , sizeof function_name);
-            type = FUNCTION_TYPE(funstart);
+            if (mstreq(function_name, STR_VARINIT))
+                flags |= TYPE_MOD_PRIVATE;
+        }
+
+        /* If the function is indeed visible, get its information */
+        if ( !(flags & (TYPE_MOD_STATIC|TYPE_MOD_PROTECTED|TYPE_MOD_PRIVATE|NAME_UNDEFINED)) )
+        {
+            string_t *function_name;
+            ident_t *p;
+            vartype_t type;
+            unsigned char num_arg, num_locals;
+
+            memcpy(  &function_name, FUNCTION_NAMEP(funstart)
+                   , sizeof function_name);
+            memcpy(&type, FUNCTION_TYPEP(funstart), sizeof(type));
             num_arg = FUNCTION_NUM_ARGS(funstart) & 0x7f;
             num_locals = FUNCTION_NUM_VARS(funstart);
 
             /* Find or make the identifier for the function */
-            p = make_shared_identifier(function_name, I_TYPE_GLOBAL, 0);
+            p = make_shared_identifier_mstr(function_name, I_TYPE_GLOBAL, 0);
             if (p->type == I_TYPE_UNKNOWN)
             {
-                p->type = I_TYPE_GLOBAL;
-                p->u.global.function = I_GLOBAL_FUNCTION_EFUN;
-                p->u.global.variable = I_GLOBAL_VARIABLE_FUN;
-                p->u.global.efun     = I_GLOBAL_EFUN_OTHER;
-                p->u.global.sim_efun = I_GLOBAL_SEFUN_OTHER;
+                init_global_identifier(p, /* bVariable: */ MY_FALSE);
                 p->next_all = all_simul_efuns;
                 all_simul_efuns = p;
             }
@@ -359,14 +376,13 @@ get_simul_efun_object (Bool require)
                     int last;
 
                     j = all_discarded_simul_efun;
-
                     while ( (j = simul_efunp[last = j].offset.func) >= 0)
                     {
                         if (num_arg != simul_efunp[j].num_arg
                          || 0 != ((simul_efunp[j].flags ^ flags) & TYPE_MOD_XVARARGS)
                            )
                             continue;
-                        if (strcmp(function_name, simul_efunp[j].name))
+                        if (!mstreq(function_name, simul_efunp[j].name))
                             continue;
 
                         /* Found one: remove it from the 'discarded' list */
@@ -374,13 +390,12 @@ get_simul_efun_object (Bool require)
                               simul_efunp[j].offset.func;
                         break;
                     }
-
                     if (j >= 0)
                         break; /* switch */
                 }
 
                 /* New simul_efun: make a new entry */
-                ref_string(function_name);
+                (void)ref_mstring(function_name);
                 j = num_simul_efun++;
                 if (num_simul_efun > num_fun)
                 {
@@ -397,10 +412,13 @@ get_simul_efun_object (Bool require)
             p->u.global.sim_efun = j;
             simul_efunp[j].name  = function_name;
             simul_efunp[j].flags = flags;
-            simul_efunp[j].type  = type;
+            simul_efunp[j].type.typeflags = type.type;
+#ifdef USE_STRUCTS
+            simul_efunp[j].type.t_struct = type.t_struct;
+#endif
 
             /* If possible, make an entry in the simul_efun table */
-            if ((size_t)j < SIZE_SEFUN_TABLE)
+            if ((size_t)j < SEFUN_TABLE_SIZE)
             {
                 simul_efun_table[j].funstart = funstart;
                 simul_efun_table[j].program = inherit_progp;
@@ -413,15 +431,15 @@ get_simul_efun_object (Bool require)
     total_simul_efun = num_fun;
     simul_efun_object = ob;
 
-    return ob;
+    return MY_TRUE;
 } /* get_simul_efun_object() */
 
 /*-------------------------------------------------------------------------*/
-char *
+string_t *
 query_simul_efun_file_name(void)
 
 /* Return the name of the primary simul_efun object.
- * Result is a shared string, but no extra reference is added.
+ * Result is a tabled string, but no extra reference is added.
  */
 
 {
@@ -484,7 +502,6 @@ count_simul_efun_refs (void)
     }
     if (simul_efun_program)
         mark_program_ref(simul_efun_program);
-
 } /* count_simul_efun_refs() */
 
 #endif /* GC_SUPPORT */
