@@ -25,6 +25,12 @@ struct simul_efun_table_s
        */
 };
 
+/* --- Defines --- */
+
+#define SEFUN_TABLE_SIZE (65536UL)
+  /* The number of entries in the sefun table, directly callable.
+   */
+
 /* --- Variables --- */
 
 extern int num_simul_efun;
@@ -36,8 +42,8 @@ extern simul_efun_table_t simul_efun_table[];
 /* --- Prototypes --- */
 
 extern void invalidate_simul_efuns (void);
-extern object_t *get_simul_efun_object(Bool require);
-extern char *query_simul_efun_file_name(void);
+extern Bool assert_simul_efun_object(void);
+extern string_t *query_simul_efun_file_name(void);
 
 #ifdef GC_SUPPORT
 extern void clear_simul_efun_refs(void);
