@@ -24,14 +24,16 @@
  * union regardless of its actual content, therefore it is of type 'p_int'.
  */
 union u {
-    char *string;
-      /* T_STRING: pointer to the first character of the string.
-       * T_SYMBOL: pointer to the shared symbol string.
-       * T_CHAR_LVALUE: pointer to the referenced character (referenced
-       *           from a T_LVALUE).
+    string_t *str;
+      /* T_STRING: pointer to the string.
+       * T_SYMBOL: pointer to the (shared) symbol string.
        * T_(PROTECTED_)STRING_RANGE_LVALUE: the target string holding
        *   the range.
        */
+    char     *charp;
+      /* T_(PROTECTED_)CHAR_LVALUE: pointer to the referenced character
+       *           (referenced from a T_LVALUE).
+       */
     p_int number;
       /* T_NUMBER: the number.
        */
@@ -45,6 +47,11 @@ union u {
        * T_(PROTECTED_)POINTER_RANGE_LVALUE: the target vector holding
        *   the range.
        */
+#ifdef USE_STRUCTS
+     struct_t *strct;
+      /* T_STRUCT: pointer to the structure instance.
+       */
+#endif
     mapping_t *map;
       /* T_MAPPING: pointer to the mapping structure.
        * T_PROTECTOR_MAPPING: TODO: ???
@@ -58,7 +65,12 @@ union u {
     callback_t *cb;
       /* T_CALLBACK: A callback structure referenced from the stack
        *   to allow proper cleanup during error recoveries. The interpreter
-       * knows how to free it, but that's all.
+       *   knows how to free it, but that's all.
+       */
+
+    void *generic;
+      /* Use read-only, this member is used to read the svalue generically
+       * as "a pointer". It is mostly used in comparisons.
        */
 
     svalue_t *lvalue;
@@ -93,17 +105,6 @@ union u {
        * it has to be referenced by pointer) and let the error_handler()
        * execute the appropriate casts.
        */
-
-#ifndef INITIALIZATION_BY___INIT
-    struct const_list_svalue_s *const_list;
-      /* Used by the LPC compiler only: when initializing global variables
-       * with static arrays, the compiler will collect the array elements
-       * in a list of const_list_ts, while letting the initializer (typed
-       * as T_LVALUE) point to the head structure of that list.
-       * Once complete, the .u.const_list is replaced by the completed .u.vec.
-       */
-#endif
-
 };
 
 /* --- struct svalue_s: the LPC data structure ---
@@ -122,16 +123,19 @@ union u {
  * are to be assigned. A special case are protected lvalues where the
  * svalue referenced by the T_LVALUE.u.lvalue is not the target svalue,
  * but instead a T_PROTECTED_xxx_LVALUE which then points to the target
- * and its protector.
+ * and its protector. Similar, indexed and range-indexed lvalues don't
+ * point to the data itself, but instead to one of the dedicated T_xxx_LVALUE
+ * types which contain the necessary information to perform the indexing
+ * operation; effectively limiting the access to the target value to
+ * the view specified by the index/range bounds.
  *
  * T_LVALUEs are also used to reference meta data, like T_ERROR_HANDLER
- * svalues.
+ * svalues. TODO: Maybe they should get the T_META type.
  */
 struct svalue_s
 {
     ph_int type;  /* Primary type information */
     union {       /* Secondary type information */
-        ph_int string_type;  /* Allocation method of the string */
         ph_int exponent;     /* Exponent of a T_FLOAT */
         ph_int closure_type; /* Type of a T_CLOSURE */
         ph_int quotes;       /* Number of quotes of a quoted array or symbol */
@@ -149,12 +153,19 @@ struct svalue_s
 };
 
 #define SVALUE_FULLTYPE(svp) ((p_int *)(svp))
-  /* Return an integer with the primary and secondary type information.
+  /* Return a pointer to an integer with the primary and secondary type
+   * information.
    */
 /* TODO: Sanity test: sizeof struct { ph_int, ph_int } <= sizeof p_int */
 
 
-/* struct svalue_s.type: Primary types */
+/* struct svalue_s.type: Primary types.
+ *   There must be no more than 16 different primary types, as some
+ *   type tables (prolang.y::hook_type_map[] for example) store
+ *   them as bitflags in shorts.
+ * TODO: Create these constants, plus the names in
+ * TODO:: interpret.c::svalue_typename[] from a descriptive _spec file.
+ */
 
 #define T_INVALID       0x0  /* empty svalue */
 #define T_LVALUE        0x1  /* a lvalue */
@@ -167,51 +178,50 @@ struct svalue_s
 #define T_CLOSURE       0x8  /* a closure */
 #define T_SYMBOL        0x9  /* a symbol */
 #define T_QUOTED_ARRAY  0xa  /* a quoted array */
+#ifdef USE_STRUCTS
+#define T_STRUCT        0xb  /* a struct */
+#endif /* USE_STRUCTS */
 
-#define T_CHAR_LVALUE                     0xb
+#define T_CHAR_LVALUE                     0xc
   /* .u.string points to the referenced character in a string */
+
   /* The following types must be used only in svalues referenced
    * by a T_LVALUE svalue.
    */
-#define T_STRING_RANGE_LVALUE             0xc /* TODO: ??? */
-#define T_POINTER_RANGE_LVALUE            0xd /* TODO: ??? */
-#define T_PROTECTED_CHAR_LVALUE           0x0e
+#define T_STRING_RANGE_LVALUE             0x0d /* TODO: ??? */
+#define T_POINTER_RANGE_LVALUE            0x0e /* TODO: ??? */
+#define T_PROTECTED_CHAR_LVALUE           0x0f
   /* A protected character lvalue */
-#define T_PROTECTED_STRING_RANGE_LVALUE   0x0f
+#define T_PROTECTED_STRING_RANGE_LVALUE   0x10
   /* A protected string range lvalue */
-#define T_PROTECTED_POINTER_RANGE_LVALUE  0x10
+#define T_PROTECTED_POINTER_RANGE_LVALUE  0x11
   /* A protected pointer/mapping range lvalue */
-#define T_PROTECTED_LVALUE                0x11
+#define T_PROTECTED_LVALUE                0x12
   /* A protected lvalue */
-#define T_PROTECTOR_MAPPING               0x12 /* TODO: ??? */
+#define T_PROTECTOR_MAPPING               0x13 /* TODO: ??? */
 
-#define T_CALLBACK                        0x13
+#define T_CALLBACK                        0x14
   /* A callback structure referenced from the stack to allow
    * proper cleanup during error recoveries. The interpreter
    * knows how to free it, but that's all.
    */
 
-#define T_ERROR_HANDLER                   0x14
+#define T_ERROR_HANDLER                   0x15
   /* Not an actual value, this is used internally for cleanup
    * operations. See the description of the error_handler() member
    * for details.
    */
 
+#define T_NULL                            0x16
+  /* Not an actual type, this is used in the efun_lpc_types[] table
+   * to encode the acceptance of '0' instead of the real datatype.
+   */
+
 #define T_MOD_SWAPPED    0x80
   /* This flag is |-ed to the swapped-out type value if the value
    * data has been swapped out.
    */
 
-
-/* T_STRING secondary information */
-
-#define STRING_MALLOC    0  /* Allocated by malloc() */
-#define STRING_VOLATILE  1  /* Static storage, must not be freed */
-#define STRING_SHARED    2  /* Allocated by the shared string module */
-  /* Constant strings are handled as volatile strings - they don't
-   * appear often enough that special treatment would be valuable.
-   */
-
 /* T_CLOSURE secondary information. */
 
   /* For closures of operators (internal machine codes), efuns and
@@ -244,8 +254,12 @@ struct svalue_s
    * the others actually point to code.
    */
 
-#define CLOSURE_LFUN            0  /* lfun in this object */
-#define CLOSURE_ALIEN_LFUN      1  /* lfun in an other object */
+#define CLOSURE_LFUN            0  /* lfun in an object */
+
+    /* Code 1 was used for ALIEN_LFUNs - some mudlibs however rely on the
+     * old code values.
+     */
+
 #define CLOSURE_IDENTIFIER      2  /* variable in this object */
 
 #define CLOSURE_PRELIMINARY     3
@@ -270,7 +284,7 @@ struct svalue_s
    * efun or simul_efun.
    */
 
-#define CLOSURE_IS_LFUN(c)        (((c)&~1) == 0)
+#define CLOSURE_IS_LFUN(c)        ((c) == CLOSURE_LFUN)
   /* TRUE if the closure is of the #'<lfun> type.
    */
 
@@ -286,6 +300,33 @@ struct svalue_s
   /* TRUE if the closure is callable.
    */
 
+/* --- The primary types in bit-flag encoding ---
+ *
+ * This flag encoding is used for the runtime typetests.
+ * Not all svalue types have a corresponding bitflag.
+ */
+
+#define TF_INVALID       (1 << T_INVALID)
+#define TF_LVALUE        (1 << T_LVALUE)
+#define TF_NUMBER        (1 << T_NUMBER)
+#define TF_STRING        (1 << T_STRING)
+#define TF_POINTER       (1 << T_POINTER)
+#define TF_OBJECT        (1 << T_OBJECT)
+#define TF_MAPPING       (1 << T_MAPPING)
+#define TF_FLOAT         (1 << T_FLOAT)
+#define TF_CLOSURE       (1 << T_CLOSURE)
+#define TF_SYMBOL        (1 << T_SYMBOL)
+#define TF_QUOTED_ARRAY  (1 << T_QUOTED_ARRAY)
+#define TF_NULL          (1 << T_NULL)
+#ifdef USE_STRUCTS
+#define TF_STRUCT        (1 << T_STRUCT)
+#endif /* USE_STRUCTS */
+
+#define TF_ANYTYPE       (~0)
+  /* This is used in the efun_lpc_types[]
+   * table to encode the acceptance of any type.
+   */
+
 /* --- Float Support --- */
 
 /* The driver uses a 48-Bit floating point format with a 32 bit mantissa
@@ -293,10 +334,8 @@ struct svalue_s
  * be well below all existing host floating point formats so that we get
  * the same accuracy on all platforms.
  *
- * The functions to encode/decode float numbers exist in two version, one
- * fast one using internal knowledge about how the compiler stores its
- * numbers, and a second portable one. To keep the implementation
- * transparent, the following macros/functions are defined:
+ * To keep the implementation transparent, the following macros/functions are
+ * defined:
  *
  *   int FLOAT_FORMAT:
  *     0 for the portable format, 1 for the fast format.
@@ -319,50 +358,6 @@ struct svalue_s
  *     Declaration of a local variable which STORE_DOUBLE needs.
  */
 
-/* --- The fast format */
-
-#if defined(atarist) || (defined(AMIGA) && defined(_DCC))
-
-#define FLOAT_FORMAT_1
-
-/* Faster routines, using inline and knowlegde about double format.
- * The exponent isn't actually in 'exponent', but that doesn't really matter
- * as long as the accesses are consistent.
- *
- * The DICE compiler for the Amiga lacks the ldexp() and frexp() functions,
- * therefore these functions here are the only way to get things done.
- *
- * STORE_DOUBLE doesn't do any rounding, but truncates off the least
- * significant bits of the mantissa that won't fit together with the exponent
- * into 48 bits. To compensate for this, we initialise the unknown bits of
- * the mantissa with 0x7fff in READ_DOUBLE . This keeps the maximum precision
- * loss of a store/read pair to the same value as rounding, while being faster
- * and being more stable.
- */
-
-static
-#ifdef atarist
-inline
-#endif
-double READ_DOUBLE(struct svalue *svalue_pnt)
-{        double tmp;
-        (*(long*)&tmp) = svalue_pnt->u.mantissa;
-        ((short*)&tmp)[2] = svalue_pnt->x.exponent;
-        ((short*)&tmp)[3] = 0x7fff;
-        return tmp;
-}
-
-#define SPLIT_DOUBLE(double_value, int_pnt) (\
-            (*(int_pnt) = ((short*)&double_value)[2]),\
-            *((long*)&double_value)\
-        )
-
-#define STORE_DOUBLE_USED
-#define STORE_DOUBLE(dest, double_value) (\
-            (dest)->u.mantissa = *((long*)&double_value),\
-            (dest)->x.exponent = ((short*)&double_value)[2]\
-        )
-#endif
 
 /* --- The portable format, used if no other format is defined */
 
@@ -370,21 +365,24 @@ double READ_DOUBLE(struct svalue *svalue_pnt)
 
 #define FLOAT_FORMAT_0
 
-#define READ_DOUBLE(svalue_pnt) ( ldexp( (double)((svalue_pnt)->u.mantissa) , \
-                (svalue_pnt)->x.exponent-31 ) )
-
+static INLINE double READ_DOUBLE(svalue_t *sv) {
+    return ldexp( (double)(sv->u.mantissa), sv->x.exponent - 31 );
+}
 /* if your machine doesn't use the exponent to designate powers of two,
-   the use of ldexp in SPLIT_DOUBLE won't work; you'll have to mulitply
+   the use of ldexp in SPLIT_DOUBLE won't work; you'll have to multiply
    with 32768. in this case */
 
-#define SPLIT_DOUBLE(double_value, int_pnt) \
-( (long)ldexp( frexp( (double_value), (int_pnt) ), 31) )
+static INLINE p_int SPLIT_DOUBLE(double doublevalue, int *int_p) {
+    return (p_int)ldexp( frexp( doublevalue, int_p ), 31);
+}
 
-#define STORE_DOUBLE_USED int __store_double_int_;
-#define STORE_DOUBLE(dest, double_value) (\
-((dest)->u.mantissa = SPLIT_DOUBLE((double_value), &__store_double_int_)),\
- (dest)->x.exponent = (short)__store_double_int_\
-)
+// STORE_DOUBLE_USED is not needed anymore and defined empty.
+#define STORE_DOUBLE_USED
+static INLINE void STORE_DOUBLE(svalue_t *dest, double doublevalue) {
+    int exponent;
+    dest->u.mantissa = SPLIT_DOUBLE(doublevalue, &exponent);
+    dest->x.exponent = (ph_int)exponent;
+}
 
 #endif /* ifndef STORE_DOUBLE */
 
@@ -412,6 +410,14 @@ double READ_DOUBLE(struct svalue *svalue_pnt)
  *   otherwise empty svalue. <value> must be a refcounted value, and
  *   its refcount is incremented.
  *
+ * void push_<type>(), void push_ref_<type>()
+ *   As the put_() macros, but <sp> is incremented once first.
+ *
+ * As a rule, all arguments must not have sideeffects!
+ *
+ * The psh_ macros are not in use (yet) - they have been renamed from push_
+ * so that the first use stands out.
+ *
  * TODO: Add push_xxx() macros, see MudOS:interpret.h. In general, get
  * TODO:: rid of the local sp/pc copies since they make error handling
  * TODO:: very difficult.
@@ -431,6 +437,13 @@ double READ_DOUBLE(struct svalue *svalue_pnt)
 #define put_array(sp,arr) \
     ( (sp)->type = T_POINTER, (sp)->u.vec = arr )
 
+#ifdef USE_STRUCTS
+#define put_ref_struct(sp,st) \
+    ( (sp)->type = T_STRUCT, (sp)->u.strct = ref_struct(st) )
+#define put_struct(sp,st) \
+    ( (sp)->type = T_STRUCT, (sp)->u.strct = st )
+#endif /* USE_STRUCTS */
+
 #define put_ref_mapping(sp,val) \
     ( (sp)->type = T_MAPPING, (sp)->u.map = ref_mapping(val) )
 #define put_mapping(sp,val) \
@@ -441,25 +454,68 @@ double READ_DOUBLE(struct svalue *svalue_pnt)
 #define put_object(sp,val) \
     ( (sp)->type = T_OBJECT, (sp)->u.ob = val )
 
-#define push_ref_object(sp,val,from) \
-    ( (sp)++, put_ref_object(sp,val,from) )
+#define put_ref_valid_object(sp,val,from) \
+    ( ((val) && !((val)->flags & O_DESTRUCTED)) \
+      ? (put_ref_object(sp,val,from), 0) : put_number(sp, 0))
+
+#define put_valid_object(sp,val) \
+    ( ((val) && !((val)->flags & O_DESTRUCTED)) \
+      ? (put_object(sp,val), 0) : put_number(sp, 0))
 
-#define put_volatile_string(sp,val) \
-    ( (sp)->type = T_STRING, (sp)->x.string_type = STRING_VOLATILE, \
-      (sp)->u.string = val )
-#define put_malloced_string(sp,val) \
-    ( (sp)->type = T_STRING, (sp)->x.string_type = STRING_MALLOC, \
-      (sp)->u.string = val )
 #define put_ref_string(sp,val) \
-    ( (sp)->type = T_STRING, (sp)->x.string_type = STRING_SHARED, \
-      (sp)->u.string = ref_string(val) )
+    ( (sp)->type = T_STRING, (sp)->u.str = ref_mstring(val) )
 #define put_string(sp,val) \
-    ( (sp)->type = T_STRING, (sp)->x.string_type = STRING_SHARED, \
-      (sp)->u.string = val )
+    ( (sp)->type = T_STRING, (sp)->u.str = val )
 
 #define put_callback(sp,val) \
     ( (sp)->type = T_CALLBACK, (sp)->u.cb = val )
 
+#define push_number(sp,num) \
+    ( (sp)++, put_number(sp, num) )
+
+#define push_float(sp,fnum) \
+    ( (sp)++, put_float(sp, fnum) )
+
+#define push_ref_array(sp,arr) \
+    ( (sp)++, put_ref_array(sp, arr) )
+#define push_array(sp,arr) \
+    ( (sp)++, put_array(sp, arr) )
+
+#ifdef USE_STRUCTS
+#define push_ref_struct(sp,st) \
+    ( (sp)++, put_ref_struct(sp, st) )
+#define push_struct(sp,st) \
+    ( (sp)++, put_struct(sp, st) )
+#endif /* USE_STRUCTS */
+
+#define psh_ref_mapping(sp,val) \
+    ( (sp)++, put_ref_mapping(sp,val) )
+#define push_mapping(sp,val) \
+    ( (sp)++, put_mapping(sp,val) )
+
+#define push_ref_object(sp,val,from) \
+    ( (sp)++, put_ref_object(sp,val,from) )
+#define push_object(sp,val) \
+    ( (sp)++, put_object(sp,val) )
+
+#define push_ref_valid_object(sp,val,from) \
+    ( (sp)++, put_ref_valid_object(sp,val,from) )
+#define push_valid_object(sp,val) \
+    ( (sp)++, put_valid_object(sp,val) )
+
+#define push_ref_string(sp,val) \
+    ( (sp)++, put_ref_string(sp,val) )
+#define push_string(sp,val) \
+    ( (sp)++, put_string(sp,val) )
+#define push_c_string(sp,txt) \
+    ( (sp)++, put_c_string(sp, txt) )
+#define push_c_n_string(sp,txt,len) \
+    ( (sp)++, put_c_n_string(sp, txt, len) )
+
+
+#define psh_callback(sp,val) \
+    ( (sp)++, put_callback(sp,val) )
+
 /* The following macros implement the dynamic cost evaluations:
  *
  *   DYN_STRING_COST(l):  increase eval_cost depending on string length <l>.
@@ -467,8 +523,8 @@ double READ_DOUBLE(struct svalue *svalue_pnt)
  *   DYN_MAPPING_COST(l): increase eval_cost depending on mapping length <l>.
  *
  * Covered so far are:
- *   F_ALLOCATE, F_ADD, F_ADD_EQ, F_VOID_ADD_EQ, F_MULTIPLY, F_MULT_EQ of
- *   strings
+ *   F_ALLOCATE, F_ADD, F_ADD_EQ, F_VOID_ADD_EQ, F_MULTIPLY,
+ *   F_MULT_EQ of strings
  *
  * TODO: Expand this to all datatypes and sizes.
  */
@@ -497,5 +553,6 @@ extern void assign_svalue_no_free(svalue_t *to, svalue_t *from);
 extern void assign_svalue(svalue_t *dest, svalue_t *v);
 extern void transfer_svalue_no_free(svalue_t *dest, svalue_t *v);
 extern void transfer_svalue(svalue_t *dest, svalue_t *v);
+extern svalue_t *pop_n_elems (int n, svalue_t *sp);
 
 #endif /* SVALUE_H__ */
