@@ -63,7 +63,7 @@
  *   free and used blocks. For the price of a larger swap file the
  *   fragmentation and search times are lower.
  *
- *   The swapper always extends the file for a new block until the more
+ *   The swapper extends the file for new blocks until more
  *   than half of the swap file is unused. At that point the swapper
  *   starts recycling the free blocks as in the compact mode, but only
  *   until the free blocks occupy only 1/4th of the swap file - then
@@ -80,37 +80,35 @@
 #include <stddef.h>
 #include <stdio.h>
 
-#define NO_REF_STRING
 #include "swap.h"
 
 #include "array.h"
 #include "backend.h"
+#include "closure.h"
 #include "comm.h"
-#include "exec.h"
 #include "gcollect.h"
 #include "interpret.h"
 #include "main.h"
 #include "mapping.h"
 #include "mempools.h"
+#include "mstrings.h"
 #include "object.h"
 #include "otable.h"
 #include "prolang.h"
 #include "random.h"
 #include "simulate.h"
 #include "simul_efun.h"
-#include "stralloc.h"
+#include "stdstrings.h"
 #include "strfuns.h"
+#ifdef USE_STRUCTS
+#include "structs.h"
+#endif /* USE_STRUCTS */
 #include "svalue.h"
 #include "wiz_list.h"
 #include "xalloc.h"
 
 #include "../mudlib/sys/debug_info.h"
 
-/* TODO: Use get_host_name() instead of gethostname() */
-#if defined(AMIGA) && !defined(__GNUC__)
-#include "hosts/amiga/socket.h"
-#endif
-
 /*-------------------------------------------------------------------------*/
 
 typedef struct swap_block_s swap_block_t;
@@ -216,17 +214,18 @@ mp_int total_bytes_swapfree = 0;
    */
 
 static unsigned char *last_variable_block;
-  /* Recursive swap during a GC: address of the last variable
+  /* Swap during a and by the GC: address of the last variable
    * block to be written.
    */
 
 static mp_int last_variable_swap_num;
-  /* Recursive swap during a GC: swap number of the last variable
+  /* Swap during a and by the GC: swap number of the last variable
    * block to be written.
    */
 
 static char *last_changed_swapped_svalue;
-  /* The last stored svalue free_swapped_svalues() had to change.
+  /* Swap during a and by the GC: The last stored svalue
+   * free_swapped_svalues() had to change.
    */
 
 
@@ -321,29 +320,21 @@ locate_out (program_t *prog)
     if (!prog)
         return MY_FALSE;
 
-    if (d_flag > 1)
-    {
-        debug_message ("%s locate_out: %lX %lX %lX %lX %lX %lX %lX %lX\n"
-            , time_stamp(),
-            (long)prog->program,
-            (long)prog->functions, (long)prog->strings,
-            (long)prog->variable_names, (long)prog->inherit,
-            (long)prog->includes,
-            (long)prog->argument_types, (long)prog->type_start);
-    }
-
 #define MAKEOFFSET(type, name) (type)&p[(char *)prog->name - (char *)prog]
 
     prog->program        = MAKEOFFSET(bytecode_p, program);
     prog->functions      = MAKEOFFSET(uint32*, functions);
     prog->function_names = MAKEOFFSET(unsigned short *, function_names);
-    prog->strings        = MAKEOFFSET(char**, strings);
-    prog->variable_names = MAKEOFFSET(variable_t *, variable_names);
+    prog->strings        = MAKEOFFSET(string_t**, strings);
+    prog->variables      = MAKEOFFSET(variable_t *, variables);
     prog->inherit        = MAKEOFFSET(inherit_t *, inherit);
+#ifdef USE_STRUCTS
+    prog->struct_defs    = MAKEOFFSET(struct_def_t *, struct_defs);
+#endif /* USE_STRUCTS */
     prog->includes       = MAKEOFFSET(include_t *, includes);
     if (prog->type_start)
     {
-        prog->argument_types = MAKEOFFSET(unsigned short *, argument_types);
+        prog->argument_types = MAKEOFFSET(vartype_t *, argument_types);
         prog->type_start = MAKEOFFSET(unsigned short *, type_start);
     }
     return MY_TRUE;
@@ -380,27 +371,19 @@ locate_in (program_t *prog)
     prog->program        = MAKEPTR(bytecode_p, program);
     prog->functions      = MAKEPTR(uint32*, functions);
     prog->function_names = MAKEPTR(unsigned short *, function_names);
-    prog->strings        = MAKEPTR(char**, strings);
-    prog->variable_names = MAKEPTR(variable_t*, variable_names);
+    prog->strings        = MAKEPTR(string_t**, strings);
+    prog->variables      = MAKEPTR(variable_t*, variables);
     prog->inherit        = MAKEPTR(inherit_t*, inherit);
+#ifdef USE_STRUCTS
+    prog->struct_defs    = MAKEPTR(struct_def_t*, struct_defs);
+#endif /* USE_STRUCTS */
     prog->includes       = MAKEPTR(include_t*, includes);
     if (prog->type_start)
     {
-        prog->argument_types = MAKEPTR(unsigned short *, argument_types);
+        prog->argument_types = MAKEPTR(vartype_t *, argument_types);
         prog->type_start     = MAKEPTR(unsigned short *, type_start);
     }
 
-    if (d_flag > 1)
-    {
-        debug_message ("%s locate_in: %lX %lX %lX %lX %lX %lX %lX %lX\n"
-            , time_stamp(),
-            (long)prog->program,
-            (long)prog->functions, (long)prog->strings,
-            (long)prog->variable_names, (long)prog->inherit,
-            (long)prog->includes,
-            (long)prog->argument_types, (long)prog->type_start);
-    }
-
     return MY_TRUE;
 
 #undef MAKEPTR
@@ -620,8 +603,8 @@ store_swap_block (void * buffer, mp_int size)
     /* Seek and write the data */
     if (fseek(swap_file, offset, 0) == -1)
     {
-        debug_message("%s Couldn't seek the swap file, errno %d, offset %ld.\n"
-                     , time_stamp(), errno, offset);
+        debug_message("%s Couldn't seek the swap file, errno %d, "
+            "offset %"PRIdMPINT".\n", time_stamp(), errno, offset);
         return -1;
     }
 
@@ -672,8 +655,8 @@ store_swap_block2 ( void * buffer1, mp_int size1
     /* Seek and write the data */
     if (fseek(swap_file, offset, 0) == -1)
     {
-        debug_message("%s Couldn't seek the swap file, errno %d, offset %ld.\n"
-                     , time_stamp(), errno, offset);
+        debug_message("%s Couldn't seek the swap file, errno %d, "
+            "offset %"PRIdMPINT".\n", time_stamp(), errno, offset);
         return -1;
     }
 
@@ -708,8 +691,9 @@ swap_program (object_t *ob)
 
     if (d_flag > 1)
     {
-        debug_message("%s Swap object %s (ref %ld)\n", time_stamp()
-                     , ob->name, ob->ref);
+        debug_message("%s Swap object %s (obj ref %"PRIdPINT
+            ", prog ref %"PRIdPINT")\n",
+            time_stamp(), get_txt(ob->name), ob->ref, ob->prog->ref);
     }
 
     prog = ob->prog;
@@ -817,10 +801,14 @@ swap_mapping_filter (svalue_t *key, svalue_t *values, void *extra)
     varblock_t *block = *((varblock_t **)extra);
 
     if (block->current)
+    {
         block = swap_svalues(key, 1, block);
+    }
 
     if (block->current)
+    {
         block = swap_svalues(values, *((p_int *)block->start), block);
+    }
 
     *((varblock_t **)extra) = block;
 } /* swap_mapping_filter() */
@@ -835,24 +823,30 @@ swap_svalues (svalue_t *svp, mp_int num, varblock_t *block)
  * The data values are added to the end of the varblock in the following
  * formats:
  *
- * swtype == (ph_int)svp->type | TYPE_MOD_SWAPPED.
+ * swtype := (ph_int)svp->type | TYPE_MOD_SWAPPED.
  *
  * STRING, SYMBOL:
- *    swtype, (ph_int)svp->x, string, \0
+ *    swtype, (ph_int)svp->x, (mp_int)mstrsize(string), string
  *
- * POINTER:
+ *    For strings, svp->x.generic is 1 for untabled strings, and 0 for
+ *    tabled string.
+ 
+ * POINTER, STRUCTS:
  *    swtype, (size_t) swapsize, (size_t)size, (wiz_list_t*) user, values...
  *
  * QUOTED_ARRAY:
  *    swtype, (ph_int)quotes, (size_t) swapsize, (size_t)size, (wiz_list_t*) user, values...
  *
+ * STRUCT:
+ *    swtype, (size_t) swapsize, (struct_type_t *)type, (wiz_list_t*)user, values...
+ *
  * MAPPING:
  *    swtype, (size_t) swapsize, (p_int)width, (p_int)size, (wiz_list_t*) user, entries...
  *
  * Opaque, NUMBER, FLOAT, OBJECT, CLOSURE
  *    svp->type, svp->x, svp->u
  *
- * Opaque are: contents of alists, empty arrays, arrays or mappings with
+ * Opaque are: contents of alists, empty arrays, structs/arrays/mappings with
  * more than one ref (this also protects against recursive data structures),
  * and mappings with closure or object keys (see the comment in the T_MAPPING
  * case for the reason).
@@ -925,6 +919,10 @@ swap_svalues (svalue_t *svp, mp_int num, varblock_t *block)
         {
 
         case T_STRING:
+            /* Use x.generic as flag whether the string is tabled or not */
+            svp->x.generic = (mstr_tabled(svp->u.str) ? 0 : 1);
+            /* FALL THROUGH */
+
         case T_SYMBOL:
           {
             mp_int len, size;
@@ -932,15 +930,17 @@ swap_svalues (svalue_t *svp, mp_int num, varblock_t *block)
             if (swapping_alist)
                 goto swap_opaque;
 
-            len = strlen(svp->u.string) + 1;
-            size = 1 + sizeof svp->x + len;
+            len = mstrsize(svp->u.str);
+            size = 1 + sizeof svp->x + sizeof(len) + len;
             CHECK_SPACE(size)
             rest -= size;
 
             *p++ = svp->type | T_MOD_SWAPPED;
             memcpy(p, &svp->x, sizeof(svp->x));
             p += sizeof svp->x;
-            memcpy(p, svp->u.string, len);
+            memcpy(p, &len, sizeof(len));
+            p += sizeof len;
+            memcpy(p, get_txt(svp->u.str), len);
             p += len;
             break;
           }
@@ -953,11 +953,11 @@ swap_svalues (svalue_t *svp, mp_int num, varblock_t *block)
             if (svp->u.vec->ref > 1 || !size || swapping_alist)
                 goto swap_opaque;
 
-            if (size > 1 && is_alist(svp->u.vec))
+            if (size > 1 && is_ordered(svp->u.vec))
                 swapping_alist = MY_TRUE;
 
             CHECK_SPACE(1 + sizeof(swapsize) + sizeof(size) + sizeof(wiz_list_t *))
-            *p++ = T_POINTER | T_MOD_SWAPPED;
+            *p++ = svp->type | T_MOD_SWAPPED;
             rest--;
             INIT_SWAPSIZE();
             ADD_TO_BLOCK(size)
@@ -968,16 +968,39 @@ swap_svalues (svalue_t *svp, mp_int num, varblock_t *block)
             break;
           }
 
+#ifdef USE_STRUCTS
+        case T_STRUCT:
+          {
+            struct_t *st = svp->u.strct;
+            size_t size;
+
+            size = struct_size(st);
+            if (st->ref > 1 || swapping_alist)
+                goto swap_opaque;
+
+            CHECK_SPACE(1 + sizeof(swapsize) + sizeof(struct_type_t *) + sizeof(wiz_list_t *))
+            *p++ = svp->type | T_MOD_SWAPPED;
+            rest--;
+            INIT_SWAPSIZE();
+            ADD_TO_BLOCK(st->type)
+            ADD_TO_BLOCK(st->user)
+            SWAP_SVALUES(st->member, size)
+            UPDATE_SWAPSIZE();
+            swapping_alist = MY_FALSE;
+            break;
+          }
+#endif /* USE_STRUCTS */
+
         case T_QUOTED_ARRAY:
           {
             size_t size;
 
             size = VEC_SIZE(svp->u.vec);
-            if (svp->u.vec->ref > 1 || !size || swapping_alist)
+            if (svp->u.vec->ref > 1 || swapping_alist)
                 goto swap_opaque;
 
             CHECK_SPACE(
-              1 + sizeof(swapsize) + sizeof svp->x.quotes +
+              1 + sizeof(swapsize)+ sizeof svp->x.quotes +
               sizeof size + sizeof(wiz_list_t *)
             )
 
@@ -996,7 +1019,7 @@ swap_svalues (svalue_t *svp, mp_int num, varblock_t *block)
         case T_MAPPING:
           {
             mapping_t *m = svp->u.map;
-            p_int size, num_values, save;
+            p_int num_entries, num_values, save;
             varblock_t *tmp;
 
             if (m->ref > 1 || swapping_alist)
@@ -1010,10 +1033,12 @@ swap_svalues (svalue_t *svp, mp_int num, varblock_t *block)
              * mapping for destructed object keys.
              */
             if (mapping_references_objects(m))
+            {
                 goto swap_opaque;
+            }
 
             CHECK_SPACE(
-              1 + sizeof(swapsize) + sizeof num_values + sizeof size + sizeof m->user
+              1 + sizeof(swapsize)+ sizeof num_values + sizeof num_entries + sizeof m->user
             )
 
             *p++ = T_MAPPING | T_MOD_SWAPPED;
@@ -1022,8 +1047,8 @@ swap_svalues (svalue_t *svp, mp_int num, varblock_t *block)
             num_values = m->num_values;
               /* The type of num_values might be wider than m->num_values. */
             ADD_TO_BLOCK(num_values);
-            size = MAP_SIZE(m);
-            ADD_TO_BLOCK(size);
+            num_entries = MAP_SIZE(m);
+            ADD_TO_BLOCK(num_entries);
             ADD_TO_BLOCK(m->user);
 
             tmp = (varblock_t *)(p + rest);
@@ -1102,10 +1127,10 @@ check_swapped_values (mp_int num, unsigned char * p)
         if (start + swapsize != p) \
         { \
             fprintf(stderr \
-                 , "--- Incorrect swapsize on check: expected %lu bytes, " \
-                   "read %lu (%p .. %p)\n" \
-                 , (unsigned long)swapsize \
-                 , (unsigned long)(p - start), start, p \
+                 , "--- Incorrect swapsize on check: expected %zu bytes, " \
+                   "read %zu (%p .. %p)\n" \
+                 , swapsize \
+                 , (size_t)(p - start), start, p \
                 ); \
             return NULL; \
         }
@@ -1154,6 +1179,25 @@ check_swapped_values (mp_int num, unsigned char * p)
             break;
           }
 
+#ifdef USE_STRUCTS
+        case T_STRUCT | T_MOD_SWAPPED:
+          {
+            wiz_list_t *user;
+            struct_type_t *stt;
+
+            GET_SWAPSIZE();
+            memcpy(&stt, p, sizeof stt);
+            p += sizeof stt;
+            memcpy(&user, p, sizeof user);
+            p += sizeof user;
+            p = check_swapped_values(struct_t_size(stt), p);
+            if (!p)
+                return NULL;
+            CHECK_SWAPSIZE();
+            break;
+          }
+#endif /* USE_STRUCTS */
+
         case T_MAPPING | T_MOD_SWAPPED:
           {
             p_int num_values;
@@ -1177,6 +1221,9 @@ check_swapped_values (mp_int num, unsigned char * p)
         case T_STRING:
         case T_SYMBOL:
         case T_POINTER:
+#ifdef USE_STRUCTS
+        case T_STRUCT:
+#endif /* USE_STRUCTS */
         case T_QUOTED_ARRAY:
         case T_MAPPING:
         case T_NUMBER:
@@ -1229,11 +1276,11 @@ dump_swapped_values (mp_int num, unsigned char * p, int indent)
         if (start + swapsize != p) \
         { \
             fprintf(stderr \
-                 , "%.*s--- Incorrect swapsize: expected %lu bytes, " \
-                   "read %lu (%p .. %p)\n" \
+                 , "%.*s--- Incorrect swapsize: expected %zu bytes, " \
+                   "read %zu (%p .. %p)\n" \
                  , indent, "               " \
-                 , (unsigned long)swapsize \
-                 , (unsigned long)(p - start), start, p \
+                 , swapsize \
+                 , (size_t)(p - start), start, p \
                 ); \
         }
 
@@ -1244,9 +1291,9 @@ dump_swapped_values (mp_int num, unsigned char * p, int indent)
 
         sv.type = *p & ~T_MOD_SWAPPED; /* get the original type */
 
-        fprintf(stderr, "%.*s%08lx (%6ld) [%3ld]: type %d"
+        fprintf(stderr, "%.*s%16p (%6zu) [%3"PRIdMPINT"]: type %d"
                       , indent, "                "
-                      , (unsigned long)p, (unsigned long)(p - block)
+                      , p, (size_t)(p - block)
                       , max_num  - num - 1
                       , sv.type
                );
@@ -1283,7 +1330,7 @@ dump_swapped_values (mp_int num, unsigned char * p, int indent)
             p += sizeof size;
             memcpy(&user, p, sizeof user);
             p += sizeof user;
-            fprintf(stderr, " array: %ld values\n", (long)size);
+            fprintf(stderr, " array: %zu values\n", size);
             p = dump_swapped_values(size, p, indent+2);
             if (!p)
                 return NULL;
@@ -1291,6 +1338,27 @@ dump_swapped_values (mp_int num, unsigned char * p, int indent)
             break;
           }
 
+#ifdef USE_STRUCTS
+        case T_STRUCT | T_MOD_SWAPPED:
+          {
+            wiz_list_t *user;
+            struct_type_t *stt;
+
+            GET_SWAPSIZE();
+            memcpy(&stt, p, sizeof stt);
+            p += sizeof stt;
+            memcpy(&user, p, sizeof user);
+            p += sizeof user;
+            fprintf(stderr, " struct '%s': %ld values\n"
+                          , get_txt(stt->name), (long)struct_t_size(stt));
+            p = dump_swapped_values(struct_t_size(stt), p, indent+2);
+            if (!p)
+                return NULL;
+            CHECK_SWAPSIZE();
+            break;
+          }
+#endif /* USE_STRUCTS */
+
         case T_MAPPING | T_MOD_SWAPPED:
           {
             p_int num_values;
@@ -1304,8 +1372,8 @@ dump_swapped_values (mp_int num, unsigned char * p, int indent)
             p += sizeof num_keys;
             memcpy(&user, p, sizeof user);
             p += sizeof user;
-            fprintf(stderr, " mapping: %ld keys, %ld values\n"
-                          , (long)num_keys, (long)num_values);
+            fprintf(stderr, " mapping: %"PRIdPINT" keys, %"PRIdPINT" values\n"
+                          , num_keys, num_values);
             p = dump_swapped_values(num_keys*(1+num_values), p, indent+2);
             if (!p)
                 return NULL;
@@ -1316,6 +1384,9 @@ dump_swapped_values (mp_int num, unsigned char * p, int indent)
         case T_STRING:
         case T_SYMBOL:
         case T_POINTER:
+#ifdef USE_STRUCTS
+        case T_STRUCT:
+#endif /* USE_STRUCTS */
         case T_QUOTED_ARRAY:
         case T_MAPPING:
         case T_NUMBER:
@@ -1394,9 +1465,9 @@ free_swapped_svalues (svalue_t *svp, mp_int num, unsigned char *p)
         if (start + swapsize != p) \
         { \
             dump_swapped_values(max_num, block, 0); \
-            fatal("svalue type %d: expected %lu bytes, read %lu (%p .. %p)\n" \
-                 , (int)svp->type, (unsigned long)swapsize \
-                 , (unsigned long)(p - start), start, p \
+            fatal("svalue type %d: expected %zu bytes, read %zu (%p .. %p)\n" \
+                 , (int)svp->type, swapsize \
+                 , (size_t)(p - start), start, p \
                 ); \
         }
 
@@ -1405,21 +1476,16 @@ free_swapped_svalues (svalue_t *svp, mp_int num, unsigned char *p)
         switch(*p)
         {
         case T_STRING | T_MOD_SWAPPED:
-            if (svp->x.string_type != STRING_SHARED)
-            {
-                if (!gc_status && svp->x.string_type == STRING_MALLOC)
-                    xfree(svp->u.string);
-                p = (unsigned char *)strchr(
-                  (char *)p + 1 + sizeof svp->x, 0
-                ) + 1;
-                break;
-            }
-            /* else fall through */
-
         case T_SYMBOL | T_MOD_SWAPPED:
-            if (!gc_status)
-                free_string(svp->u.string);
-            p = (unsigned char *)strchr((char *)p + 1 + sizeof svp->x, 0) + 1;
+            {
+                mp_int strsize;
+                p += 1 + sizeof(svp->x);
+                memcpy(&strsize, p, sizeof(strsize));
+                p += sizeof(strsize) + strsize;
+
+                if (!gc_status)
+                    free_mstring(svp->u.str);
+            }
             break;
 
         case T_QUOTED_ARRAY | T_MOD_SWAPPED:
@@ -1436,6 +1502,20 @@ free_swapped_svalues (svalue_t *svp, mp_int num, unsigned char *p)
             CHECK_SWAPSIZE();
             break;
 
+#ifdef USE_STRUCTS
+        case T_STRUCT | T_MOD_SWAPPED:
+          {
+            p += 1;
+            GET_SWAPSIZE();
+            p += sizeof(struct_type_t *) + sizeof(wiz_list_t *);
+            p =
+              free_swapped_svalues(svp->u.strct->member, struct_size(svp->u.strct), p);
+            struct_free_empty(svp->u.strct);
+            CHECK_SWAPSIZE();
+            break;
+          }
+#endif /* USE_STRUCTS */
+
         case T_MAPPING | T_MOD_SWAPPED:
           {
             /* beware: a mapping can get unswappable when it is entered
@@ -1476,7 +1556,7 @@ free_swapped_svalues (svalue_t *svp, mp_int num, unsigned char *p)
             /* the garbage collector replaces closures bound to destructed
              * objects by F_UNDEF
              */
-            if (svp->x.closure_type == CLOSURE_EFUN + F_UNDEF)
+            if (is_undef_closure(svp)) /* this shouldn't happen */
             {
               if ( memcmp(p+1, &svp->x, sizeof svp->x)
                 || memcmp(p+1+sizeof svp->x, &svp->u, sizeof svp->u))
@@ -1494,6 +1574,9 @@ free_swapped_svalues (svalue_t *svp, mp_int num, unsigned char *p)
         case T_STRING:
         case T_SYMBOL:
         case T_POINTER:
+#ifdef USE_STRUCTS
+        case T_STRUCT:
+#endif /* USE_STRUCTS */
         case T_QUOTED_ARRAY:
         case T_MAPPING:
         case T_NUMBER:
@@ -1546,7 +1629,12 @@ swap_variables (object_t *ob)
 
     if (gc_status)
     {
-        /* Complete the previous swap operation */
+        /* During a GC, the swapper is called in close swap-in/swap-out
+         * sequences. To minimized interaction with the allocator,
+         * the swap out only has to check if svalues changed due to
+         * the GC; otherwise it is sufficient to just pretend to swap
+         * out and free the associated memory.
+         */
         num_variables = ob->prog->num_variables;
         last_changed_swapped_svalue = NULL;
         (void)free_swapped_svalues(
@@ -1558,8 +1646,9 @@ swap_variables (object_t *ob)
             if (fseek(swap_file, last_variable_swap_num + sizeof(p_int), 0) ==
                                                                             -1)
             {
-                fatal("Couldn't seek the swap file, errno %d, offset %ld.\n",
-                      errno, last_variable_swap_num + sizeof(p_int));
+                fatal("Couldn't seek the swap file, errno %d, offset %"
+                    PRIdMPINT".\n",
+                    errno, last_variable_swap_num + sizeof(p_int));
             }
             if (fwrite(
                   last_variable_block,
@@ -1574,7 +1663,21 @@ swap_variables (object_t *ob)
         ob->variables = (svalue_t *)(last_variable_swap_num | 1);
         ob->flags |= O_SWAPPED;
 
+#ifdef CHECK_OBJECT_STAT
+        if (check_object_stat)
+        {
+            fprintf(stderr, "DEBUG: OSTAT: (%ld:%ld) swapout( %p '%s') gc %d "
+                "vars : %ld -> (%ld:%ld)\n",
+                tot_alloc_object, tot_alloc_object_size, ob, 
+                ob->name ? get_txt(ob->name) : "<null>",
+                num_variables, (long)(num_variables * sizeof (svalue_t)),
+                tot_alloc_object, 
+                tot_alloc_object_size - (num_variables * sizeof (svalue_t))
+                );
+        }
+#endif
         tot_alloc_object_size -= num_variables * sizeof (svalue_t);
+
         return MY_TRUE;
     }
 
@@ -1589,8 +1692,8 @@ swap_variables (object_t *ob)
         if (swapfile_size <= swap_num)
             fatal("Attempt to swap in from beyond the end of the swapfile.\n");
         if (fseek(swap_file, swap_num, 0) == -1)
-            fatal("Couldn't seek the swap file, errno %d, offset %ld.\n",
-                  errno, swap_num);
+            fatal("Couldn't seek the swap file, errno %d, offset %"
+                PRIdPINT".\n", errno, swap_num);
         if (fread(&num_variables, sizeof num_variables, 1, swap_file)
             != 1)
         {
@@ -1644,6 +1747,19 @@ swap_variables (object_t *ob)
     total_vb_bytes_swapped += total_size - sizeof total_size;
     mb_free(mbSwap);
     xfree(ob->variables);
+#ifdef CHECK_OBJECT_STAT
+    if (check_object_stat)
+    {
+        fprintf(stderr, "DEBUG: OSTAT: (%ld:%ld) swapout( %p '%s') %d "
+            "vars : %ld -> (%ld:%ld)\n",
+            tot_alloc_object, tot_alloc_object_size, ob, 
+            ob->name ? get_txt(ob->name) : "<null>",
+            num_variables, (long)(num_variables * sizeof (svalue_t)),
+            tot_alloc_object, 
+            tot_alloc_object_size - (num_variables * sizeof (svalue_t))
+            );
+    }
+#endif
     tot_alloc_object_size -= num_variables * sizeof (svalue_t);
 
     /* Mark the variables as swapped */
@@ -1660,9 +1776,12 @@ Bool
 swap (object_t *ob, int mode)
 
 /* Swap the object <ob> according to <mode>:
+ *
  *   <mode> & 0x01: swap program
  *   <mode> & 0x02: swap variables
  *
+ * The same flags are returned by load_ob_from_swap().
+ *
  * Result is TRUE if all requested swaps succeeded.
  */
 
@@ -1673,9 +1792,13 @@ swap (object_t *ob, int mode)
         return MY_FALSE;
 
     if (mode & 2)
+    {
         result = swap_variables(ob) && result;
+    }
     if (mode & 1)
+    {
         result = swap_program(ob) && result;
+    }
     return result;
 } /* swap() */
 
@@ -1721,9 +1844,9 @@ read_unswapped_svalues (svalue_t *svp, mp_int num, unsigned char *p)
         if (start + swapsize != p) \
         { \
             dump_swapped_values(max_num, block, 0); \
-            fatal("svalue type %d: expected %lu bytes, read %lu (%p .. %p)\n" \
-                 , (int)svp->type, (unsigned long)swapsize \
-                 , (unsigned long)(p - start), start, p \
+            fatal("svalue type %d: expected %zu bytes, read %zu (%p .. %p)\n" \
+                 , (int)svp->type, swapsize \
+                 , (size_t)(p - start), start, p \
                 ); \
         }
 
@@ -1737,10 +1860,13 @@ read_unswapped_svalues (svalue_t *svp, mp_int num, unsigned char *p)
         case T_STRING | T_MOD_SWAPPED:
         case T_SYMBOL | T_MOD_SWAPPED:
           {
-            char *s;
+            string_t *s;
+            mp_int    len;
 
             memcpy(&svp->x, p, sizeof svp->x);
             p += sizeof svp->x;
+            memcpy(&len, p, sizeof len);
+            p += sizeof len;
             if (gc_status)
             {
                 svp->type = T_NUMBER;
@@ -1748,15 +1874,13 @@ read_unswapped_svalues (svalue_t *svp, mp_int num, unsigned char *p)
             else
             {
                 if (svp->type == T_STRING
-                 && svp->x.string_type != STRING_SHARED)
+                 && svp->x.generic != 0)
                 {
-                    s = string_copy((char *)p);
-                    svp->x.string_type = STRING_MALLOC;
-                      /* Might have been STRING_VOLATILE */
+                    s = new_n_mstring((char *)p, len);
                 }
                 else
                 {
-                    s = make_shared_string((char *)p);
+                    s = new_n_tabled((char *)p, len);
                 }
 
                 if (!s)
@@ -1765,9 +1889,9 @@ read_unswapped_svalues (svalue_t *svp, mp_int num, unsigned char *p)
                     return NULL;
                 }
 
-                svp->u.string = s;
+                svp->u.str = s;
             }
-            p = (unsigned char *)strchr((char *)p, '\0') + 1;
+            p += len;
             break;
           }
 
@@ -1815,6 +1939,48 @@ read_unswapped_svalues (svalue_t *svp, mp_int num, unsigned char *p)
             break;
           }
 
+#ifdef USE_STRUCTS
+        case T_STRUCT | T_MOD_SWAPPED:
+          {
+            wiz_list_t *user;
+            struct_t *st;
+            struct_type_t *stt;
+
+            GET_SWAPSIZE();
+            memcpy(&stt, p, sizeof stt);
+            p += sizeof stt;
+            memcpy(&user, p, sizeof user);
+            p += sizeof user;
+            current_object->user = user;
+            st = struct_new(stt);
+            (void)deref_struct_type(stt); /* just reactivate the old ref */
+            svp->u.strct = st;
+            if (!st)
+            {
+                clear_svalues(svp, num + 1);
+                return NULL;
+            }
+            p = read_unswapped_svalues(st->member, struct_size(st), p);
+            if (!p)
+            {
+                clear_svalues(svp + 1, num);
+                return NULL;
+            }
+            CHECK_SWAPSIZE();
+#ifdef GC_SUPPORT
+            if (gc_status == gcCountRefs)
+            {
+                /* Pretend that this memory block was already existing
+                 * in the clear phase.
+                 */
+                clear_memory_reference(st);
+                st->ref = 0;
+            }
+#endif
+            break;
+          }
+#endif /* USE_STRUCTS */
+
         case T_MAPPING | T_MOD_SWAPPED:
           {
             mapping_t *m;
@@ -1838,50 +2004,38 @@ read_unswapped_svalues (svalue_t *svp, mp_int num, unsigned char *p)
                  * because the garbage collector runs with
                  * malloc_privilege == MALLOC_SYSTEM .
                  */
-                struct condensed_mapping *cm;
+                mapping_cond_t *cm;
                 mp_int size;
                 svalue_t *data, *svp2;
 
-                size =
-                  sizeof *cm + num_keys * sizeof(svalue_t) * (1 + num_values);
-                cm = (struct condensed_mapping *)
-                  ( (char *)xalloc(size) + size - sizeof *cm );
-                m = (mapping_t *)xalloc(sizeof *m);
-                cm->string_size = 0;
-                cm->misc_size = num_keys * sizeof(svalue_t);
-                m->hash = 0;
-                m->condensed = cm;
-                m->num_values = num_values;
-                m->ref = 1;
-                m->user = user;
-                user->mapping_total +=
-                    sizeof *m + sizeof(char*) + size + sizeof(char*);
-                num_mappings++;
+                m = allocate_cond_mapping(user, num_keys, num_values);
                 svp->u.map = m;
-                svp2 = CM_MISC(cm);
-                size = cm->misc_size;
-                data = (svalue_t *)((char *)svp2 - size);
-                while ( (size -= sizeof(svalue_t)) >= 0) {
-                    data -= num_values;
-                    p = read_unswapped_svalues(--svp2, 1, p);
-                    p = read_unswapped_svalues(data, num_values, p);
-                }
-#ifdef GC_SUPPORT
-                if (gc_status == gcCountRefs)
+                if (m->cond)
                 {
-                    /* Pretend that this memory block already existing
-                     * in the clear phase.
-                     */
-                    clear_memory_reference(m);
-                    clear_memory_reference(
-                      (char *)CM_MISC(cm) -
-                        cm->misc_size * (m->num_values + 1)
-                    );
-                    m->ref = 0;
-                }
+                    cm = m->cond;
+
+                    svp2 = &(cm->data[0]);
+                    size = cm->size;
+                    data = COND_DATA(cm, 0, num_values);
+                    while ( --size >= 0) {
+                        p = read_unswapped_svalues(svp2++, 1, p);
+                        p = read_unswapped_svalues(data, num_values, p);
+                        data += num_values;
+                    }
+#ifdef GC_SUPPORT
+                    if (gc_status == gcCountRefs)
+                    {
+                        /* Pretend that this memory block was already existing
+                         * in the clear phase.
+                         */
+                        clear_memory_reference(m);
+                        clear_memory_reference(cm);
+                        m->ref = 0;
+                    }
 #endif
+                }
             }
-            else /* !gcStatus */
+            else
             {
                 mp_int i;
                 wiz_list_t *save;
@@ -1924,6 +2078,9 @@ read_unswapped_svalues (svalue_t *svp, mp_int num, unsigned char *p)
         case T_STRING:
         case T_SYMBOL:
         case T_POINTER:
+#ifdef USE_STRUCTS
+        case T_STRUCT:
+#endif /* USE_STRUCTS */
         case T_QUOTED_ARRAY:
         case T_MAPPING:
         case T_NUMBER:
@@ -1950,7 +2107,7 @@ read_unswapped_svalues (svalue_t *svp, mp_int num, unsigned char *p)
 
 /*-------------------------------------------------------------------------*/
 static void
-dummy_handler(char * fmt UNUSED, ...)
+dummy_handler(const char * fmt UNUSED, ...)
 
 /* Dummy error handler for array allocations in a swap-in.
  */
@@ -1979,6 +2136,8 @@ load_ob_from_swap (object_t *ob)
  *
  *    result & 0x01: program swapped in
  *    result & 0x02: variables swapped in
+ *
+ * The same flags are accepted by swap() as argument.
  */
 
 {
@@ -1999,12 +2158,12 @@ load_ob_from_swap (object_t *ob)
         if (swapfile_size <= swap_num)
             fatal("Attempt to swap in from beyond the end of the swapfile.\n");
         if (fseek(swap_file, swap_num, 0) == -1)
-            fatal("Couldn't seek the swap file, errno %d, offset %ld.\n",
-                  errno, swap_num);
+            fatal("Couldn't seek the swap file, errno %d, offset %"
+                PRIdPINT".\n", errno, swap_num);
         if (d_flag > 1)
         {
-            debug_message("%s Unswap object %s (ref %ld)\n", time_stamp()
-                         , ob->name, ob->ref);
+            debug_message("%s Unswap object %s (ref %"PRIdPINT")\n", 
+                time_stamp(), get_txt(ob->name), ob->ref);
         }
 
         /* The size of the program is unkown, so read first part to
@@ -2058,18 +2217,18 @@ load_ob_from_swap (object_t *ob)
         svalue_t *variables;
         object_t dummy;
         object_t *save_current = current_object;
-        void (*save_handler)(char *, ...);
+        void (*save_handler)(const char *, ...);
 
         swap_num &= ~1;
         if (swapfile_size <= swap_num)
             fatal("Attempt to swap in from beyond the end of the swapfile.\n");
         if (fseek(swap_file, swap_num, 0) == -1)
-            fatal("Couldn't seek the swap file, errno %d, offset %ld.\n",
-                  errno, swap_num);
+            fatal("Couldn't seek the swap file, errno %d, offset %"
+                PRIdPINT".\n", errno, swap_num);
         if (d_flag > 1)
         {
             debug_message("%s Unswap variables of %s\n", time_stamp()
-                         , ob->name);
+                         , get_txt(ob->name));
         }
 
         /* Read the size of the block from the file */
@@ -2107,12 +2266,28 @@ load_ob_from_swap (object_t *ob)
         {
             ob->variables = variables;
             result |= 2;
+#ifdef CHECK_OBJECT_STAT
+            if (check_object_stat)
+            {
+                fprintf(stderr, "DEBUG: OSTAT: (%ld:%ld) swapin( %p '%s') %d "
+                    "vars : %ld -> (%ld:%ld)\n",
+                    tot_alloc_object, tot_alloc_object_size, ob, 
+                    ob->name ? get_txt(ob->name) : "<null>",
+                    ob->prog->num_variables,
+                    (long)(ob->prog->num_variables * sizeof (svalue_t)),
+                    tot_alloc_object, tot_alloc_object_size + 
+                      (ob->prog->num_variables * sizeof (svalue_t))
+                    );
+            }
+#endif
             tot_alloc_object_size += ob->prog->num_variables * sizeof (svalue_t);
 
             if (gc_status)
             {
-                /* Called from within the GC - free later.
-                 * Also keep the swap-block
+                /* During a GC, the swapper is called in close swap-in/swap-out
+                 * sequences. To minimized interaction with the allocator,
+                 * the memory blocks are kept for now, and freed during the
+                 * 'swap out'.
                  */
                 last_variable_block = block;
                 last_variable_swap_num = swap_num;
@@ -2128,6 +2303,17 @@ load_ob_from_swap (object_t *ob)
         else
         {
             /* Out of memory */
+#ifdef CHECK_OBJECT_STAT
+            if (check_object_stat)
+            {
+                fprintf(stderr, "DEBUG: OSTAT: (%ld:%ld) swapin( %p '%s') %d "
+                    "vars failed\n",
+                    tot_alloc_object, tot_alloc_object_size, ob, 
+                    ob->name ? get_txt(ob->name) : "<null>",
+                    ob->prog->num_variables);
+            }
+#endif
+
             mb_free(mbSwap);
             result |= -0x80;
         }
@@ -2163,8 +2349,8 @@ load_line_numbers_from_swap (program_t *prog)
     if (swapfile_size <= swap_num)
         fatal("Attempt to swap in from beyond the end of the swapfile.\n");
     if (fseek(swap_file, swap_num, 0) == -1)
-        fatal("Couldn't seek the swap file, errno %d, offset %ld.\n",
-              errno, swap_num);
+        fatal("Couldn't seek the swap file, errno %d, offset %"
+            PRIdPINT".\n", errno, swap_num);
     if (fread((char *)&tmp_numbers, sizeof tmp_numbers, 1, swap_file) != 1) {
         fatal("Couldn't read the swap file.\n");
     }
@@ -2225,8 +2411,8 @@ remove_prog_swap (program_t *prog, Bool load_line_numbers)
         linenumbers_t tmp_lines;
 
         if (fseek(swap_file, swap_num + prog->total_size, 0 ) == -1)
-            fatal("Couldn't seek the swap file, errno %d, offset %ld.\n",
-                  errno, swap_num);
+            fatal("Couldn't seek the swap file, errno %d, offset %"
+                PRIdPINT".\n", errno, swap_num);
         if (fread(&tmp_lines, sizeof tmp_lines, 1, swap_file) != 1)
         {
             fatal("Couldn't read the swap file.\n");
@@ -2292,12 +2478,12 @@ swap_status (strbuf_t *sbuf)
  */
 
 {
-    /* maximum seen so far: 10664 var blocks swapped,    5246112 bytes */
-    strbuf_addf(sbuf, "%6ld prog blocks swapped,%10ld bytes\n"
-                      "%6ld prog blocks unswapped,%8ld bytes\n"
-                      "%6ld var blocks swapped,%11ld bytes\n"
-                      "%6ld free blocks in swap,%10ld bytes\n"
-                      "Swapfile size:%23ld bytes\n"
+    /* maximum seen so far: 28574 var blocks swapped,   32754860 bytes */
+    strbuf_addf(sbuf, "%10"PRIdMPINT" prog blocks swapped, %13"PRIdMPINT" bytes\n"
+                      "%10"PRIdMPINT" prog blocks unswapped,%12"PRIdMPINT" bytes\n"
+                      "%10"PRIdMPINT" var blocks swapped,%15"PRIdMPINT" bytes\n"
+                      "%10"PRIdMPINT" free blocks in swap,%14"PRIdMPINT" bytes\n"
+                      "Swapfile size:%31"PRIdMPINT" bytes\n"
                 , num_swapped - num_unswapped
                 , total_bytes_swapped - total_bytes_unswapped
                 , num_unswapped, total_bytes_unswapped
@@ -2305,11 +2491,11 @@ swap_status (strbuf_t *sbuf)
                 , num_swapfree, total_bytes_swapfree
                 , swapfile_size
     );
-    strbuf_addf(sbuf, "Total reused space:%18ld bytes\n\n"
+    strbuf_addf(sbuf, "Total reused space:%26"PRIdMPINT" bytes\n\n"
                     , total_swap_reused);
     strbuf_addf(sbuf
-               , "Swap: searches: %5ld average search length: %3.1f\n"
-                 "Free: searches: %5ld average search length: %3.1f\n"
+               , "Swap: searches: %10ld average search length: %3.1f\n"
+                 "Free: searches: %10ld average search length: %3.1f\n"
                , swap_num_searches
                , (double)swap_total_searchlength /
                  ( swap_num_searches ? swap_num_searches : 1 )
@@ -2317,9 +2503,9 @@ swap_status (strbuf_t *sbuf)
                , (double)swap_free_searchlength /
                  ( swap_free_searches ? swap_free_searches : 1 )
     );
-    strbuf_addf(sbuf, "Overhead: %ld blocks using %ld bytes.\n"
-               , num_swap_structs, num_swap_structs * sizeof(swap_block_t)
-               );
+    strbuf_addf(sbuf, "Overhead: %"PRIdMPINT" blocks using %"
+        PRIdMPINT" bytes.\n",
+        num_swap_structs, num_swap_structs * sizeof(swap_block_t) );
     strbuf_addf(sbuf, "Mode: %s - Freespace recycling: %s\n"
                , swap_compact_mode ? "compact" : "non-compact"
                , (recycle_free_space || !swap_compact_mode) ? "on" : "off"
