@@ -28,7 +28,7 @@
  * TODO:: And it surely still contains bugs.
  *---------------------------------------------------------------------------
  */
-
+ 
 #include <sys/types.h>
 #include <errno.h>
 #include <netdb.h>
@@ -91,8 +91,10 @@ time_t time(time_t *);
 #    define SIGTERM 15
 #endif
 
-#ifdef _AIX
-typedef unsigned long length_t;  /* *sigh* */
+#if defined(_AIX)
+typedef unsigned long length_t;
+#elif defined(__INTEL_COMPILER) || defined (__GNUC__)
+typedef socklen_t length_t;
 #else
 typedef int length_t;
 #endif
@@ -450,7 +452,7 @@ execute (char *buf, long buflen, char *status, int *sockets)
     int quoted;
 
     quoted = 0;
-    status[0] = ERQ_E_FORKFAIL; /* Good default */;
+    status[0] = ERQ_E_FORKFAIL; /* Good default */
     status[1] = 0;
 
     if (buflen >= sizeof argbuf)
@@ -570,7 +572,7 @@ execute (char *buf, long buflen, char *status, int *sockets)
         status[1] = errno;
         return 0;
     }
-
+    
     return pid;
 } /* execute() */
 
@@ -588,7 +590,7 @@ count_sigcld (int sig)
 #ifdef __MWERKS__
 #    pragma unused(sig)
 #endif
-
+        
 #ifndef HAVE_WAITPID
     static volatile int calling_signal = 0;     /* Mutex */
     static volatile int call_signal_again = 0;  /* Number of pending calls */
@@ -776,7 +778,7 @@ get_subserver (void)
         }
 
         /* Set up the child and put it into the subserver list */
-
+        
         close(sockets[0]);
         child->u.c.pid = pid;
         child->socket = sockets[1];
@@ -837,7 +839,7 @@ start_subserver (long server_num, long seed)
     for (;;) /* The Loop (tm) */
     {
         /* select() for data */
-
+        
         if (child)
         {
             FD_SET(child_sockets[1], &readfds);
@@ -957,12 +959,12 @@ start_subserver (long server_num, long seed)
             }
             childs_waited_for++;
         } /* wait for zombies */
-
+        
         if (!FD_ISSET(0, &readfds))
             continue;
 
         /* Receive the new request from the master ERQ */
-
+        
         do
             num = readn(0, header, 9);
         while (num == -1 && errno == EINTR);
@@ -979,7 +981,7 @@ start_subserver (long server_num, long seed)
         request = header[8];
 
         /* ... and the rest of the data */
-
+        
         if (msglen > 0)
         {
             if (msglen > sizeof(buf))
@@ -1010,13 +1012,13 @@ start_subserver (long server_num, long seed)
         }
 
         /* What does the ERQ want? */
-
+        
         switch(request)
         {
         case ERQ_RLOOKUP:
           {
             /* Lookup ip -> name */
-
+            
             struct hostent *hp;
 
             /* handle stays in header[4..7] */
@@ -1035,7 +1037,7 @@ start_subserver (long server_num, long seed)
                 sleep(7);
                 hp = gethostbyaddr(buf, 4, AF_INET);
             }
-
+            
             if (hp)
             {
                 /* Send the address and the name */
@@ -1057,7 +1059,7 @@ start_subserver (long server_num, long seed)
         case ERQ_LOOKUP:
           {
             /* Lookup name -> ip */
-
+            
             struct hostent *hp;
 
             /* handle stays in header[4..7] */
@@ -1070,7 +1072,6 @@ start_subserver (long server_num, long seed)
             hp = gethostbyname(buf);
             if (!hp && h_errno == TRY_AGAIN)
             {
-                /* DNS needs more time */
 #if ERQ_DEBUG > 2
                  fprintf(stderr, "%s: ERQ_LOOKUP '%s': trying again in 3 sec.\n", time_stamp(), buf);
 #endif
@@ -1177,7 +1178,7 @@ start_subserver (long server_num, long seed)
             /* Execute a program, wait for its termination and
              * return the exit status.
              */
-
+             
             pid_t pid1, pid2;
 
             header[8] = CHILD_FREE;
@@ -1229,7 +1230,7 @@ start_subserver (long server_num, long seed)
              * information to the ERQ so that further communcation
              * is possible.
              */
-
+             
 #if ERQ_DEBUG > 0
             if (child) {
                 fprintf(stderr, "%s ERQ_SPAWN: busy\n", time_stamp());
@@ -1550,7 +1551,7 @@ main (int argc, char **argv)
     (void)signal(SIGCLD, (RETSIGTYPE(*)())count_sigcld);
 
     /* The main loop */
-
+    
     for (subserver = 0;;)
     {
         int still_corked; /* Determines the select() wait time */
@@ -1608,7 +1609,7 @@ main (int argc, char **argv)
         current_time = time(NULL);
 
         /* Kill off idle free children */
-
+        
         {
             time_t expired;
 
@@ -1731,7 +1732,7 @@ main (int argc, char **argv)
 
 #ifdef ERQ_OPEN_UDP
             /* Check for data received on UDP sockets for the gamedriver */
-
+            
             for (next_child = udp_sockets; (child = next_child); )
             {
                 length_t length;
@@ -1769,7 +1770,7 @@ main (int argc, char **argv)
 #ifdef ERQ_OPEN_TCP
 
             /* Exchange data between the TCP sockets and the driver */
-
+            
             for (next_child = tcp_sockets; (child = next_child); )
             {
                 int length;
@@ -1809,7 +1810,7 @@ main (int argc, char **argv)
                     }
 
                     /* Inform the driver that there is data pending */
-
+                    
                     replyheader[12] = ERQ_OK;
                     write_32(replyheader, 17+sizeof(child->u.s.ticket));
                     write_32(replyheader+4, ERQ_HANDLE_KEEP_HANDLE);
@@ -1830,7 +1831,7 @@ main (int argc, char **argv)
                 if (cnt <= 0)
                 {
                     /* No data there - EOF or error */
-
+                    
                     if (!cnt)
                     {
                         replyheader[8] = ERQ_EXITED;
@@ -1856,7 +1857,7 @@ main (int argc, char **argv)
                     /* We got data - send it to the driver (but make
                      * sure not to overrun it).
                      */
-
+                    
                     length = cnt;
 #if 0
                     /* Update the "data pending" strategy */
@@ -1964,7 +1965,7 @@ main (int argc, char **argv)
             {
 
                                       /* ----- Send a signal or data ----- */
-            case ERQ_SEND:
+            case ERQ_SEND:  
             case ERQ_KILL:
               {
                 long n;
@@ -2151,7 +2152,7 @@ main (int argc, char **argv)
             case ERQ_OPEN_UDP:
               {
                 /* Open a UDP socket */
-
+                
                 subserver = 0; /* ready for new commands */
                 write_32(header, 10);
                 do {
@@ -2214,7 +2215,7 @@ main (int argc, char **argv)
                     memcpy(child->u.s.ticket.c, ticket.c, sizeof ticket);
 
                     /* Compose the answer to the driver */
-
+                    
                     header[3] = 17 + sizeof ticket;
                     memcpy(header+8, header+4, 4);
                     write_32(header+4, ERQ_HANDLE_KEEP_HANDLE);
@@ -2292,7 +2293,7 @@ main (int argc, char **argv)
                     }
 
                     /* Got the socket, now create the child */
-
+                    
                     child = get_socket_child();
                     child->socket = s;
                     FD_SET(child->socket,&current_fds);
@@ -2310,7 +2311,7 @@ main (int argc, char **argv)
                     memcpy(child->u.s.ticket.c, ticket.c, sizeof ticket);
 
                     /* Compose the answer to the driver */
-
+                    
                     header[3] = 17 + sizeof ticket;
                     memcpy(header+8, header+4, 4);
                     write_32(header+4, ERQ_HANDLE_KEEP_HANDLE);
@@ -2421,7 +2422,7 @@ main (int argc, char **argv)
               {
 
                 /* Accept a connection from a socket */
-
+                
                 subserver = 0; /* ready for new commands */
                 write_32(header, 10);
                 do {
