@@ -10,8 +10,10 @@
 
 #include "defs.h"
 
-#ifdef _AIX
-typedef unsigned long length_t;  /* *sigh* */
+#if defined(_AIX)
+typedef unsigned long length_t;
+#elif defined(__INTEL_COMPILER) || defined (__GNUC__)
+typedef socklen_t length_t;
 #else
 typedef int length_t;
 #endif
@@ -74,7 +76,7 @@ free_socket (socket_t *sp)
         free(qp);
         qp = qnext;
     }
-
+    
     free(sp);
 } /* free_socket() */
 
@@ -252,7 +254,7 @@ read_socket (socket_t *sp, int rw)
     int num;
 
     num = 0;
-
+    
     XPRINTF((stderr, "%s Read socket %p (fd %d, type %d) , rw %d\n"
                    , time_stamp(), sp, sp->fd, sp->type, rw));
 
@@ -279,7 +281,7 @@ read_socket (socket_t *sp, int rw)
     case SOCKET_UDP:
       {
         /* Read the data message and pass it on */
-
+        
         struct sockaddr_in addr;
         length_t len;
 
@@ -298,7 +300,7 @@ read_socket (socket_t *sp, int rw)
             }
             if (num <= 0)
                 break;
-            replyn(sp->handle, 1, 4,
+            replyn(sp->handle, 1, 4, 
                 r, sizeof(r),
                 (char *) &addr.sin_addr.s_addr, 4,
                 (char *) &addr.sin_port, 2,
@@ -314,7 +316,7 @@ read_socket (socket_t *sp, int rw)
             /* Our opened TCP socket finally connected */
 
             char r[] = { ERQ_OK };
-
+            
             XPRINTF((stderr, "%s   Socket connected.\n"
                            , time_stamp() ));
             sp->type = SOCKET_CONNECTED;
@@ -333,7 +335,7 @@ read_socket (socket_t *sp, int rw)
     case SOCKET_CONNECTED:
       {
         /* Read data from the stream and pass it on */
-
+        
         char r_err[] = { ERQ_STDERR };
         char r_out[] = { ERQ_STDOUT };
 
@@ -352,7 +354,7 @@ read_socket (socket_t *sp, int rw)
             if (num <= 0)
                 break;
             replyn(sp->handle, 1, 2,
-              (sp->type==SOCKET_STDERR) ? r_err : r_out, 1,
+              (sp->type==SOCKET_STDERR) ? r_err : r_out, 1, 
               buf, num);
         } while(1);
         break;
@@ -398,7 +400,7 @@ read_socket (socket_t *sp, int rw)
     {
         /* We got EOF when reading data - the socket is no longer needed.
          */
-
+         
         XPRINTF((stderr, "%s   EOF when reading data.\n", time_stamp()));
         close_socket(sp);
         return 1;
@@ -435,7 +437,7 @@ erq_send (char *mesg, int msglen)
     char *data;
 
     /* Find the socket identified in the message */
-
+    
     if (msglen < 9+TICKET_SIZE)
     {
         char r_arglen[] = { ERQ_E_ARGLENGTH };
@@ -468,7 +470,7 @@ erq_send (char *mesg, int msglen)
     case SOCKET_CONNECTED:
       {
         /* Send the data to the stream */
-
+        
         do
             num = write(sp->fd, data, msglen);
         while (num == -1 && errno == EINTR);
@@ -519,7 +521,7 @@ erq_send (char *mesg, int msglen)
     } /* switch(socket->type) */
 
     /* Return the success of the action */
-
+    
     if (num == msglen)
     {
         char r_ok[] = { ERQ_OK, 0 };
@@ -629,7 +631,7 @@ erq_open_udp (char *mesg, int msglen)
     memcpy(&addr.sin_port, mesg + 9, 2);
 
     /* Open and initialise the socket */
-
+    
     if ((fd=socket(AF_INET, SOCK_DGRAM, 0)) < 0
      || fcntl(fd, F_SETFD, 1) < 0
      ||        fcntl(fd, F_SETFL, O_NONBLOCK) < 0
@@ -784,7 +786,7 @@ erq_accept (char *mesg, int msglen)
     {
         char r_ok[] = { ERQ_OK };
         replyn(sp->handle = get_handle(mesg), 1, 4,
-            r_ok, sizeof(r_ok),
+            r_ok, sizeof(r_ok), 
             (char *) &addr.sin_addr.s_addr, 4,
             (char *) &addr.sin_port, 2,
             (char *) &sp->ticket, TICKET_SIZE);
@@ -824,7 +826,7 @@ erq_auth (char *mesg, int msglen)
  * If the connection can't be completed immediately (EINPROGRESS), the
  * socket is set to SOCKET_WAIT_AUTH.
  */
-
+ 
 {
     struct sockaddr_in addr;
     auth_t *ap;
@@ -838,7 +840,7 @@ erq_auth (char *mesg, int msglen)
 
         memcpy(&sh, mesg+13, sizeof(sh)); remote_port = ntohs(sh);
         memcpy(&sh, mesg+15, sizeof(sh)); local_port = ntohs(sh);
-
+        
         memcpy(&addr.sin_addr.s_addr, mesg+9, 4);
         addr.sin_family = AF_INET;
         addr.sin_port = htons(AUTH_PORT);
